@@ -1,8 +1,8 @@
 #!/bin/bash
 #
-# Compile the Linux kernel under Syd under Perf.
+# Compile Linux kernel under Syd under Perf.
 #
-# Copyright 2024 Ali Polatel <alip@chesswob.org>
+# Copyright 2024, 2025 Ali Polatel <alip@chesswob.org>
 #
 # SPDX-License-Identifier: GPL-3.0
 
@@ -25,7 +25,7 @@ DIR="$(mktemp -d --tmpdir=/tmp syd-lnx.XXXXXXXXXX)"
 [[ -d "${DIR}" ]] || exit 2
 
 CWD=$(readlink -f ${PWD})
-trap "mv '${DIR}'/git/perf.data* '${CWD}' &>/dev/null && rm -rf '${DIR}'" INT TERM EXIT
+trap "mv '${DIR}'/linux/perf.data* '${CWD}' &>/dev/null && rm -rf '${DIR}'" INT TERM EXIT
 set -ex
 
 pushd "${DIR}"
@@ -37,4 +37,4 @@ export SYD_PID_FN=/tmp/syd.pid
 "${PERF}" "${@}" -- \
     "${SYD}" -q -p"${PROF}" -pP -m 'allow/all+/***' -mlock:on \
         -- \
-        sh -cex 'make defconfig && make -j$(nproc) && make clean'
+        sh -cex 'make defconfig && make -j && make clean'
