@@ -357,7 +357,7 @@ fn lookup_addr(addr: IpAddr) -> Result<String, Errno> {
         getnameinfo(
             addr.as_ptr(),
             addr.len(),
-            host_buf.as_mut_ptr(),
+            host_buf.as_mut_ptr().cast(),
             host_buf.len() as socklen_t,
             ptr::null_mut(),
             0,
@@ -385,7 +385,7 @@ fn lookup_addr(addr: IpAddr) -> Result<String, Errno> {
     }
 
     // SAFETY: On success, `host_buf` contains a valid null-terminated string.
-    let cstr = unsafe { CStr::from_ptr(host_buf.as_ptr()) };
+    let cstr = unsafe { CStr::from_ptr(host_buf.as_ptr().cast()) };
     let name = cstr.to_string_lossy().into_owned();
     Ok(name)
 }
@@ -1833,7 +1833,7 @@ Can you help me?
 Send bug reports to {PKG_AUTHORS}.
 Attaching poems encourages consideration tremendously.
 
-Homepage: https://sydbox.exherbolinux.org
+Homepage: https://sydbox.exherbo.org
 Repository: https://gitlab.exherbo.org/sydbox/
 "#,
     );
