@@ -11,6 +11,7 @@
 
 use std::{
     collections::BTreeMap,
+    os::fd::OwnedFd,
     sync::{Arc, Condvar, Mutex},
 };
 
@@ -19,12 +20,12 @@ use nix::{errno::Errno, unistd::Pid};
 use serde::{ser::SerializeMap, Serializer};
 
 use crate::{
-    elf::ExecutableFile, fs::CanonicalPath, hook::RemoteProcess, ScmpNotifReq, SydArch,
-    SydMemoryMap, SydSigSet,
+    elf::ExecutableFile, fs::CanonicalPath, hook::RemoteProcess, proc::proc_status_open,
+    ScmpNotifReq, SydArch, SydMemoryMap, SydSigSet,
 };
 
 /// Metadata on a blocking syscall invocation
-#[derive(Copy, Clone, Debug)]
+#[derive(Debug)]
 pub struct SysInterrupt {
     /// The thread group ID
     pub tgid: Pid,
@@ -35,6 +36,8 @@ pub struct SysInterrupt {
     /// True if `SA_RESTART` is ignored
     /// (e.g. due to a socket timeout).
     pub ignore_restart: bool,
+    /// FD to /proc/$pid/status for efficient access.
+    pub status_fd: OwnedFd,
 }
 
 /// This is the data type used to handle syscall interrupts.
@@ -102,13 +105,17 @@ impl SysInterrupt {
         tgid: Pid,
         handler: Pid,
         ignore_restart: bool,
-    ) -> Self {
-        Self {
+    ) -> Result<Self, Errno> {
+        #[allow(clippy::cast_possible_wrap)]
+        let status_fd = proc_status_open(Pid::from_raw(request.pid as libc::pid_t))?;
+
+        Ok(Self {
             tgid,
             handler,
             request,
             ignore_restart,
-        }
+            status_fd,
+        })
     }
 }
 
