@@ -88,6 +88,9 @@ pub const ENV_FORCE_TTY: &str = "SYD_FORCE_TTY";
 /// The environment variable to quiet TTY output.
 pub const ENV_QUIET_TTY: &str = "SYD_QUIET_TTY";
 
+/// The environment variable to hold the PTY child fd (used internally).
+pub const ENV_PTY_FD: &str = "SYD_PTY_FD";
+
 /// The environment variable to hold the IPC epoll fd (used internally).
 pub const ENV_IPC_POLL_FD: &str = "SYD_IPC_POLL_FD";
 /// The environment variable to hold the IPC UNIX socket (used internally).
@@ -251,8 +254,8 @@ pub const MON_GRACE_TIME: Duration = Duration::from_millis(250);
 pub const MON_CYCLE_TIME: Duration = Duration::from_millis(25);
 
 /// Specify the cycle period of the interrupt thread.
-/// Defaults to 5 milliseconds.
-pub const INT_CYCLE_TIME: Duration = Duration::from_millis(5);
+/// Defaults to 10 milliseconds.
+pub const INT_CYCLE_TIME: Duration = Duration::from_millis(10);
 
 /// Specify the cycle period of the aes thread.
 /// Defaults to 250 milliseconds.
@@ -267,8 +270,8 @@ pub const EMU_STACK_SIZE: usize = 208 * 1024;
 pub const MON_STACK_SIZE: usize = 96 * 1024;
 
 /// Stack size for the syscall interrupt thread.
-/// Defaults to 256k.
-pub const INT_STACK_SIZE: usize = 256 * 1024;
+/// Defaults to 128k.
+pub const INT_STACK_SIZE: usize = 128 * 1024;
 
 /// Stack size for the IPC thread.
 /// Defaults to 256k.
@@ -457,22 +460,70 @@ pub const UNSAFE_ENV: &[&[u8]] = &[
 
 /// Default ioctl allow/denylist.
 /// false -> allow, true -> deny.
+/*
+ * Quotes from Landlock ioctl access helpers
+ * is_masked_device_ioctl and is_masked_device_ioctl_compat:
+ *
+ * 1. FIOCLEX, FIONCLEX, FIONBIO and FIOASYNC manipulate the FD's
+ *    close-on-exec and the file's buffered-IO and async flags.
+ *    These operations are also available through fcntl(2),
+ *    and are unconditionally permitted in Landlock.
+ * 2. FIOQSIZE queries the size of a regular file, directory, or link.
+ *    We still permit it, because it always returns -ENOTTY for
+ *    other file types.
+ * 3. FIFREEZE and FITHAW freeze and thaw the file system which the
+ *    given file belongs to. Requires CAP_SYS_ADMIN.
+ *    These commands operate on the file system's superblock rather
+ *    than on the file itself. The same operations can also be
+ *    done through any other file or directory on the same file
+ *    system, so it is safe to permit these.
+ * 4. FS_IOC_FIEMAP queries information about the allocation of
+ *    blocks within a file.
+ *    This IOCTL command only makes sense for regular files and is
+ *    not implemented by devices. It is harmless to permit.
+ * 5. FIGETBSZ queries the file system's block size for a file or
+ *    directory.
+ *    This command operates on the file system's superblock rather
+ *    than on the file itself. The same operation can also be done
+ *    through any other file or directory on the same file system,
+ *    so it is safe to permit it.
+ * 6. FICLONE, FICLONERANGE and FIDEDUPERANGE make files share
+ *    their underlying storage ("reflink") between source and
+ *    destination FDs, on file systems which support that.
+ *    These IOCTL commands only apply to regular files
+ *    and are harmless to permit for device files.
+ * 7. FS_IOC_GETFSUUID and FS_IOC_GETFSSYSFSPATH both operate on
+ *    the file system superblock, not on the specific file, so
+ *    these operations are available through any other file on the
+ *    same file system as well.
+ * 8. FIONREAD, FS_IOC_GETFLAGS, FS_IOC_SETFLAGS, FS_IOC_FSGETXATTR and
+ *    FS_IOC_FSSETXATTR are forwarded to device implementations.
+ * 9. file_ioctl() commands (FIBMAP, FS_IOC_RESVSP, FS_IOC_RESVSP64,
+ *    FS_IOC_UNRESVSP, FS_IOC_UNRESVSP64 and FS_IOC_ZERO_RANGE) are
+ *    forwarded to device implementations, so not permitted.
+ * 10. FICLONE is permitted, same as in the non-compat variant.
+ * 11. CONFIG_X86_64: FS_IOC_RESVSP_32, FS_IOC_RESVSP64_32, FS_IOC_UNRESVSP_32,
+ *     FS_IOC_UNRESVSP64_32, FS_IOC_ZERO_RANGE_32: not blanket-permitted,
+ *     for consistency with their non-compat variants.
+ * 12. FS_IOC32_GETFLAGS, FS_IOC32_SETFLAGS are forwarded to their device
+ *     implementations.
+ */
 pub const DEFAULT_IOCTL: &[(u64, bool)] = &[
     (0x5451, false),     // FIOCLEX
     (0x5450, false),     // FIONCLEX
     (0x5421, false),     // FIONBIO
     (0x5452, false),     // FIOASYNC
     (0x5460, false),     // FIOQSIZE
-    (0xc0045877, false), // FIFREEZE
-    (0xc0045878, false), // FITHAW
-    (0xc020660b, false), // FS_IOC_FIEMAP
-    (0x2, false),        // FIGETBSZ
+    (0xc0045877, true),  // FIFREEZE: CAP_SYS_ADMIN!
+    (0xc0045878, true),  // FITHAW:   ditto!
+    (0xc020660b, true),  // FS_IOC_FIEMAP: Leaks on-disk layout.
+    (0x2, true),         // FIGETBSZ: ditto!
     (0x40049409, false), // FICLONE
     (0x4020940d, false), // FICLONERANGE
     (0xc0189436, false), // FIDEDUPERANGE
-    (0x80111500, false), // FS_IOC_GETFSUUID
-    (0x80811501, false), // FS_IOC_GETFSSYSFSPATH
-    (0x1, true),         // FIBMAP
+    (0x80111500, true),  // FS_IOC_GETFSUUID: Leaks on-disk layout.
+    (0x80811501, true),  // FS_IOC_GETFSSYSFSPATH: ditto!
+    (0x1, true),         // FIBMAP: ditto!
     (0x4b4d, true),      // KDSETKEYCODE
     (0x4b4e, true),      // KDSIGACCEPT
     (0x5423, true),      // TIOCSETD
@@ -488,7 +539,7 @@ pub const DEFAULT_IOCTL: &[(u64, bool)] = &[
 
 /// Off sandboxing profile.
 /// Used as shorthand to turn all sandboxing off.
-pub const PROFILE_OFF: &[&str] = &["sandbox/all:off", "sandbox/net,mem,pid,force,tpe:off"];
+pub const PROFILE_OFF: &[&str] = &["sandbox/all:off", "sandbox/net,mem,pid,pty,force,tpe:off"];
 
 /// Libsyd helper sandboxing profile.
 /// Turns all sandboxing off and set sandbox lock to Exec.
@@ -633,6 +684,8 @@ pub const PROFILE_PALUDIS: &[&str] = &[
     "name/host:localhost", // only effective when combined with unshare/uts:1
     "lock:exec",         // esandbox
     "log/verbose:false", // verbose logging is intended for malware analysis.
+    "mask^",             // do not inherit the default mask-list from linux profile.
+    "sandbox/pty:off",    // Paludis creates a PTY on its own.
     "trace/memory_access:0", // fallback to unsafe proc_pid_mem(5) if cross memory attach fails.
     "trace/allow_safe_bind:true",
     "trace/allow_safe_kcapi:true",
@@ -647,6 +700,7 @@ pub const PROFILE_PALUDIS: &[&str] = &[
     "trace/allow_unsafe_memory:true",
     "trace/allow_unsafe_msgsnd:true",
     "trace/allow_unsafe_nice:true",
+    "trace/allow_unsafe_nocookie:true", // syscookies are a mitigation against compromised Syd.
     "trace/allow_unsafe_nomseal:true", // mseal(2) is a mitigation against compromised Syd.
     "trace/allow_unsafe_nopie:true",
     "trace/allow_unsafe_open_path:true",
@@ -705,7 +759,6 @@ pub const PROFILE_PALUDIS: &[&str] = &[
     "allow/read,stat+/proc/stat",
     "allow/read,stat+/proc/uptime",
     "allow/read,stat+/proc/version",
-    "mask-/proc/version", // override mask in linux profile.
     // SAFETY: We allow relaxed stat access (/proc wide).
     // This must be combined with trace/allow_unsafe_magiclinks:1.
     "allow/read,stat+/proc/[0-9]*/stat",
@@ -846,6 +899,12 @@ pub const PROFILE_LINUX: &[&str] = &[
     "allow/read,stat+/proc/self/task/[0-9]*/statm",
     "allow/read,stat+/proc/self/status",
     "allow/read,stat+/proc/self/task/[0-9]*/status",
+    // SAFETY: Mask global info/stat paths as an extra layer such as:
+    // /proc/buddyinfo, /proc/cpuinfo, /proc/meminfo, /proc/pagetypeinfo,
+    // /proc/slabinfo, /proc/vmallocinfo, /proc/zoneinfo,
+    // /proc/diskstats, /proc/schedstat, /proc/stat, /proc/vmstat etc.
+    "mask+/proc/*info*/***:/dev/null:/var/empty",
+    "mask+/proc/*stat*/***:/dev/null:/var/empty",
     // SAFETY: We provide an empty file in place of /proc/cmdline
     // for compatibility rather than denying outright. In any case,
     // syd does not leak the contents of this file to the sandbox
@@ -856,14 +915,51 @@ pub const PROFILE_LINUX: &[&str] = &[
     // This is consistent with uname(2) restrictions.
     "allow/read,stat+/proc/version",
     "mask+/proc/version",
+    // SAFETY: Further masking for sa{f,n}e defaults.
+    "mask+/proc/interrupts",
+    "mask+/proc/kcore",
+    "mask+/proc/keys",
+    "mask+/proc/acpi/***:/dev/null:/var/empty",
+    "mask+/proc/asound/***:/dev/null:/var/empty",
+    "mask+/proc/bus/***:/dev/null:/var/empty",
+    "mask+/proc/driver/***:/dev/null:/var/empty",
+    // /proc/dynamic_debug
+    // SAFETY: Provide wider debug glob for future-safety.
+    "mask+/proc/*debug*/***:/dev/null:/var/empty",
+    "mask+/proc/fs/***:/dev/null:/var/empty",
+    "mask+/proc/irq/***:/dev/null:/var/empty",
+    // /proc/latency_stats, /proc/timer_list, /proc/timer_stats etc.
+    "mask+/proc/latency_*/***:/dev/null:/var/empty",
+    "mask+/proc/timer_*/***:/dev/null:/var/empty",
+    "mask+/proc/*_stats/***:/dev/null:/var/empty",
+    "mask+/proc/pressure/***:/dev/null:/var/empty",
+    // /proc/schedstat, /proc/sched_debug etc.
+    "mask+/proc/sched*/***:/dev/null:/var/empty",
+    "mask+/proc/scsi/***:/dev/null:/var/empty",
     // SAFETY: Modification of /proc/sys/kernel/modprobe requires no
     // capabilities (can cause arbitrary code to be inserted into the
     // kernel via a replacement modprobe)!
     // https://forums.grsecurity.net/viewtopic.php?f=7&t=2522
-    "mask+/proc/sys/kernel/modprobe",
+    //
+    // As of 3.36.0 we extend this to protect more such as:
+    // /proc/sys, /proc/sysvipc, /proc/sysrq-trigger etc.
+    "mask+/proc/sys*/***:/dev/null:/var/empty",
+    "mask+/proc/tty/***:/dev/null:/var/empty",
+    // Done /proc masking, move on!
+    //
+    // Wait, there is more!
+    // SAFETY: Mask critical /sys paths as an extra layer.
+    // Extra layer because this profile allows no access
+    // to the sysfs directory otherwise.
+    "mask+/sys/dev/***:/dev/null:/var/empty",
+    "mask+/sys/devices/***:/dev/null:/var/empty",
+    "mask+/sys/firmware/***:/dev/null:/var/empty",
+    "mask+/sys/fs/***:/dev/null:/var/empty",
     // SAFETY: Provide an empty file in place of /sys/kernel/notes.
     // See: https://lwn.net/Articles/962782/
-    "mask+/sys/kernel/notes",
+    // As of 3.36.0, we extend it to the directory recursively.
+    "mask+/sys/kernel/***:/dev/null:/var/empty",
+    // Done /sys masking, move on!
     "allow/stat,chdir,readdir+/proc/self",
     "allow/stat,chdir,readdir+/proc/thread-self",
     "allow/read,stat+/proc/self/comm",
@@ -927,6 +1023,7 @@ pub const PROFILE_LINUX: &[&str] = &[
     "allow/read,stat+/etc/locale.conf",
     "allow/read,stat+/etc/localtime",
     "allow/read,stat+/etc/machine-id",
+    // SAFETY: Do not leak machine ID to the sandbox process.
     "mask+/etc/machine-id",
     "allow/read,stat+/etc/man_db.conf",
     "allow/read,stat+/etc/nanorc",
@@ -973,6 +1070,7 @@ pub const PROFILE_TTY: &[&str] = &[
     "allow/read,stat,write,ioctl+${SYD_TTY}",
     // PTY handling
     "ioctl/allow+0x5410",     // TIOCSPGRP
+    "ioctl/allow+0x5401",     // TCGETS: isatty(3), tcgetattr(3) etc.
     "ioctl/allow+0x5432",     // TCGETX
     "ioctl/allow+0x5433",     // TCSETX
     "ioctl/allow+0x5434",     // TCSETXF
@@ -1902,7 +2000,6 @@ pub const SAFE_SYSCALLS: &[&str] = &[
 ];
 
 /// System calls allowed for emulator threads.
-// TODO: Further restrict open & socket calls.
 pub const EMU_SYSCALLS: &[&str] = &[
     "_llseek",
     "accept4",
@@ -1931,8 +2028,8 @@ pub const EMU_SYSCALLS: &[&str] = &[
     "fstat64",
     "fstatfs",
     "fstatfs64",
-    "ftruncate",
-    "ftruncate64",
+    //"ftruncate", protected with syscall argument cookies.
+    //"ftruncate64", ditto.
     // "futex", See: FUTEX_SYSCALLS for futex family.
     "get_robust_list",
     "getdents64",
@@ -1990,14 +2087,14 @@ pub const EMU_SYSCALLS: &[&str] = &[
     "setxattrat",
     "sigaltstack",
     "sigprocmask",
-    "socket",
+    //"socket", protected with syscall argument cookies.
     "splice",
     "symlinkat",
     "tee",
     "tgkill",
     "tkill",
-    "truncate",
-    "truncate64",
+    //"truncate", protected with syscall argument cookies.
+    //"truncate64", ditto.
     "umask",
     "uname",
     "waitid",
@@ -2044,6 +2141,8 @@ pub const INT_SYSCALLS: &[&str] = &[
     "gettid",
     "munmap", // memory deallocation is allowed but allocation is not.
     "nanosleep",
+    "lseek",   // seek in /proc/$pid/status file.
+    "_llseek", // ditto.
     "read",
     "restart_syscall",
     "rt_sigprocmask",
@@ -2100,7 +2199,7 @@ pub const AES_SYSCALLS: &[&str] = &[
     //"fcntl", See: AES_FCNTL_OPS for allowed fcntl operations.
     //"fcntl64",
     "fremovexattr",
-    "ftruncate",
+    "ftruncate", // TODO: protect with syscall argument cookies.
     "ftruncate64",
     // "futex", See: FUTEX_SYSCALLS for futex family.
     "get_robust_list",
