@@ -15,6 +15,9 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
+// SAFETY: This module has been liberated from unsafe code!
+#![forbid(unsafe_code)]
+
 //! Set of functions to manage parsing ELF files
 
 use std::{
@@ -23,7 +26,7 @@ use std::{
     fmt::Write as FmtWrite,
     fs::File,
     io::{self, BufWriter, ErrorKind, Read, Seek, SeekFrom, Write},
-    os::fd::{AsRawFd, FromRawFd},
+    os::fd::AsFd,
     process::{Command, Stdio},
 };
 
@@ -186,7 +189,7 @@ impl fmt::Display for ElfError {
         match self {
             ElfError::BadMagic => write!(f, "Invalid ELF magic number"),
             ElfError::Malformed => write!(f, "Malformed ELF header"),
-            ElfError::IoError(e) => write!(f, "I/O error: {}", e),
+            ElfError::IoError(e) => write!(f, "I/O error: {e}"),
         }
     }
 }
@@ -339,7 +342,7 @@ impl ExecutableFile {
     ///
     /// This function ensures safety by handling all potential I/O
     /// errors gracefully.
-    pub fn is_script_file<F: AsRawFd>(fd: Option<&F>, path: &XPath) -> Result<bool, ElfError> {
+    pub fn is_script_file<Fd: AsFd>(fd: Fd, path: &XPath) -> Result<bool, ElfError> {
         let (mut file, _) = safe_open_file(fd, path)?;
         let mut shebang = [0u8; 2];
         file.read_exact(&mut shebang)?;
@@ -963,9 +966,7 @@ pub fn disasm(
 
     // Create a temporary file using nix.
     let (fd, path) = mkstemp("/tmp/syd_objdumpXXXXXX")?;
-
-    // SAFETY: mkstemp returns a valid FD.
-    let mut file = BufWriter::new(unsafe { File::from_raw_fd(fd) });
+    let mut file = BufWriter::new(File::from(fd));
 
     // Write the machine code to the temporary file.
     file.write_all(machine_code).map_err(|err| err2no(&err))?;
@@ -1105,7 +1106,7 @@ pub fn disasm_x86(
         let raw_bytes = &machine_code[start_pos..end_pos];
         let hex_str = raw_bytes
             .iter()
-            .map(|b| format!("{:02x}", b))
+            .map(|b| format!("{b:02x}"))
             .collect::<Vec<_>>()
             .join("");
 
@@ -1177,7 +1178,7 @@ fn disasm_armv7(machine_code: &[u8]) -> Result<Vec<Instruction>, Errno> {
                 if (old_offset as usize) < machine_code.len() {
                     let b = machine_code[old_offset as usize];
                     instructions.push(Instruction {
-                        hex: format!("{:02x}", b),
+                        hex: format!("{b:02x}"),
                         op: Some("null".to_string()),
                     });
                     // Manually consume one byte from the reader to move on.
@@ -1243,7 +1244,7 @@ fn disasm_armv8(machine_code: &[u8]) -> Result<Vec<Instruction>, Errno> {
                 if (old_offset as usize) < machine_code.len() {
                     let b = machine_code[old_offset as usize];
                     instructions.push(Instruction {
-                        hex: format!("{:02x}", b),
+                        hex: format!("{b:02x}"),
                         op: Some("null".to_string()),
                     });
                     // Manually consume one byte from the reader to move on.
@@ -1330,7 +1331,7 @@ pub fn disasm_riscv64(machine_code: &[u8]) -> Result<Vec<Instruction>, Errno> {
                                 // => “null” for just the first byte, skip 1.
                                 let b = machine_code[offset];
                                 instructions.push(Instruction {
-                                    hex: format!("{:02x}", b),
+                                    hex: format!("{b:02x}"),
                                     op: Some("null".to_string()),
                                 });
                                 offset += 1;
@@ -1341,7 +1342,7 @@ pub fn disasm_riscv64(machine_code: &[u8]) -> Result<Vec<Instruction>, Errno> {
                         // Not enough bytes to try 32-bit => “null” for first byte.
                         let b = machine_code[offset];
                         instructions.push(Instruction {
-                            hex: format!("{:02x}", b),
+                            hex: format!("{b:02x}"),
                             op: Some("null".to_string()),
                         });
                         offset += 1;
@@ -1353,7 +1354,7 @@ pub fn disasm_riscv64(machine_code: &[u8]) -> Result<Vec<Instruction>, Errno> {
             // 3) If fewer than 2 bytes remain, we can’t decode 16-bit => “null” each leftover byte.
             let b = machine_code[offset];
             instructions.push(Instruction {
-                hex: format!("{:02x}", b),
+                hex: format!("{b:02x}"),
                 op: Some("null".to_string()),
             });
             offset += 1;
