@@ -7,7 +7,7 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    hash::{BuildHasher, Hasher},
+    hash::BuildHasher,
     io::{IoSlice, Read},
     os::fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd, OwnedFd, RawFd},
 };
@@ -1054,46 +1054,6 @@ pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
     Ok(nwrite)
 }
 
-/// Avoid duplicate hashing while using `HashSet` with u64 keys.
-pub struct NoHasher {
-    value: u64,
-}
-
-impl Hasher for NoHasher {
-    fn write(&mut self, _bytes: &[u8]) {
-        unreachable!("NoHasher should only be used for u64 keys");
-    }
-
-    fn write_u64(&mut self, i: u64) {
-        self.value = i;
-    }
-
-    fn finish(&self) -> u64 {
-        self.value
-    }
-}
-
-/// A builder for creating instances of `NoHasher`.
-#[derive(Clone)]
-pub struct NoHasherBuilder;
-
-impl Default for NoHasherBuilder {
-    fn default() -> Self {
-        Self
-    }
-}
-
-impl BuildHasher for NoHasherBuilder {
-    type Hasher = NoHasher;
-
-    fn build_hasher(&self) -> Self::Hasher {
-        NoHasher { value: 0 }
-    }
-}
-
-// A `HashSet` with no hashers.
-// pub type NoHashSet = HashSet<u64, NoHasherBuilder>;
-
 /// Returns a reference to the AT_RANDOM buffer, which is 16 bytes long.
 pub fn get_at_random() -> &'static [u8; 16] {
     // SAFETY: In libc we trust.
