@@ -89,7 +89,7 @@ pub(crate) fn sysenter_exec(
     };
 
     // Read remote path.
-    let (mut path, _, _) =
+    let (mut path, _, _, empty_path) =
         // SAFETY: We will validate the PidFd afterwards.
         unsafe { process.read_path(sandbox, data.args, arg, false, None) }?;
     if !process.is_alive() {
@@ -99,13 +99,15 @@ pub(crate) fn sysenter_exec(
     // Call sandbox access checker if Exec sandboxing is on.
     let caps = sandbox.getcaps(Capability::CAP_EXEC | Capability::CAP_TPE);
     if caps.contains(Capability::CAP_EXEC) {
+        let hide = !(empty_path && arg.flags.contains(SysFlags::EMPTY_PATH))
+            && sandbox.enabled(Capability::CAP_STAT);
         sandbox_path(
             None,
             sandbox,
             process,
             path.abs(),
             Capability::CAP_EXEC,
-            sandbox.enabled(Capability::CAP_STAT),
+            hide,
             syscall_name,
         )?;
     }
