@@ -13,6 +13,7 @@ use std::{
 };
 
 use libseccomp::ScmpNotifResp;
+use memchr::arch::all::is_prefix;
 use nix::{errno::Errno, fcntl::OFlag};
 
 use crate::{
@@ -27,6 +28,7 @@ use crate::{
     sandbox::{Action, Capability, IntegrityError},
     warn,
 };
+
 const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
 const MAP_ANONYMOUS: u64 = libc::MAP_ANONYMOUS as u64;
 
@@ -90,7 +92,7 @@ fn syscall_mem_handler(
     }
 
     if (exec || force || tpe || restrict_stack)
-        && matches!(name, "mmap" | "mmap2")
+        && is_prefix(name.as_bytes(), b"mmap")
         && req.data.args[2] & PROT_EXEC != 0
         && req.data.args[3] & MAP_ANONYMOUS == 0
     {
@@ -138,13 +140,13 @@ fn syscall_mem_handler(
                 if verbose {
                     error!("ctx": "trusted_path_execution",
                         "err": "library load from untrusted path blocked",
-                        "sys": request.syscall, "path": &path,
+                        "sys": name, "path": &path,
                         "req": &request,
                         "tip": "move the library to a safe location or use `sandbox/tpe:off'");
                 } else {
                     error!("ctx": "trusted_path_execution",
                         "err": "library load from untrusted path blocked",
-                        "sys": request.syscall, "path": &path,
+                        "sys": name, "path": &path,
                         "pid": request.scmpreq.pid,
                         "tip": "move the library to a safe location or use `sandbox/tpe:off'");
                 }
@@ -195,14 +197,14 @@ fn syscall_mem_handler(
                             if verbose {
                                 error!("ctx": "check_lib",
                                     "err": "library load with executable stack blocked",
-                                    "sys": request.syscall, "path": path.abs(),
+                                    "sys": name, "path": path.abs(),
                                     "tip": "configure `trace/allow_unsafe_stack:1'",
                                     "lib": format!("{exe}"),
                                     "req": &request);
                             } else {
                                 error!("ctx": "check_lib",
                                     "err": "library load with executable stack blocked",
-                                    "sys": request.syscall, "path": path.abs(),
+                                    "sys": name, "path": path.abs(),
                                     "tip": "configure `trace/allow_unsafe_stack:1'",
                                     "lib": format!("{exe}"),
                                     "pid": request.scmpreq.pid);
@@ -232,12 +234,12 @@ fn syscall_mem_handler(
                         if !matches!(action, Action::Allow | Action::Filter) {
                             if verbose {
                                 warn!("ctx": "verify_lib", "act": action,
-                                    "sys": request.syscall, "path": path.abs(),
+                                    "sys": name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
-                                    "sys": request.syscall, "req": &request);
+                                    "sys": name, "req": &request);
                             } else {
                                 warn!("ctx": "verify_lib", "act": action,
-                                    "sys": request.syscall, "path": path.abs(),
+                                    "sys": name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "pid": request.scmpreq.pid);
                             }
@@ -258,13 +260,13 @@ fn syscall_mem_handler(
                         if verbose {
                             error!("ctx": "verify_lib",
                                 "err": format!("system error during library checksum calculation: {errno}"),
-                                "sys": request.syscall, "path": path.abs(),
+                                "sys": name, "path": path.abs(),
                                 "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                 "req": &request);
                         } else {
                             error!("ctx": "verify_lib",
                                 "err": format!("system error during library checksum calculation: {errno}"),
-                                "sys": request.syscall, "path": path.abs(),
+                                "sys": name, "path": path.abs(),
                                 "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                 "pid": request.scmpreq.pid);
                         }
@@ -279,13 +281,13 @@ fn syscall_mem_handler(
                             if sandbox.verbose {
                                 error!("ctx": "verify_lib", "act": action,
                                     "err": format!("library checksum mismatch: {found} is not {expected}"),
-                                    "sys": request.syscall, "path": path.abs(),
+                                    "sys": name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "req": &request);
                             } else {
                                 error!("ctx": "verify_lib", "act": action,
                                     "err": format!("library checksum mismatch: {found} is not {expected}"),
-                                    "sys": request.syscall, "path": path.abs(),
+                                    "sys": name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "pid": request.scmpreq.pid);
                             }
@@ -335,12 +337,12 @@ fn syscall_mem_handler(
             if mem_act != Action::Filter {
                 if verbose {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                        "sys": request.syscall, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
+                        "sys": name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
                         "tip": "increase `mem/vm_max'",
                         "req": &request);
                 } else {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                        "sys": request.syscall, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
+                        "sys": name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
                         "tip": "increase `mem/vm_max'",
                         "pid": request.scmpreq.pid);
                 }
@@ -371,12 +373,12 @@ fn syscall_mem_handler(
                 if mem_act != Action::Filter {
                     if verbose {
                         warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                            "sys": request.syscall, "mem_max": mem_max,
+                            "sys": name, "mem_max": mem_max,
                             "tip": "increase `mem/max'",
                             "req": &request);
                     } else {
                         warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                            "mem_max": mem_max,
+                            "sys": name, "mem_max": mem_max,
                             "tip": "increase `mem/max'",
                             "pid": request.scmpreq.pid);
                     }
