@@ -126,7 +126,7 @@ pub(crate) mod unlink;
 /// xattr handlers
 pub(crate) mod xattr;
 
-use std::{borrow::Cow, os::fd::RawFd};
+use std::borrow::Cow;
 
 use libseccomp::ScmpNotifResp;
 use memchr::memmem;
@@ -137,8 +137,8 @@ use nix::{
 };
 
 use crate::{
-    fs::{CanonicalPath, FileInfo, FileType},
-    hook::{PathArgs, RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
+    fs::{to_valid_fd, CanonicalPath, FileInfo, FileType},
+    hook::{PathArgs, RemoteProcess, SysArg, UNotifyEventRequest},
     notice,
     path::{XPath, XPathBuf},
     sandbox::{Action, Capability, SandboxGuard},
@@ -159,7 +159,7 @@ pub(crate) fn sandbox_path(
 ) -> Result<(), Errno> {
     // Check for chroot.
     if sandbox.is_chroot() {
-        return Err(Errno::ENOENT);
+        return Err(if hide { Errno::ENOENT } else { Errno::EACCES });
     }
 
     // Check enabled capabilities.
@@ -363,18 +363,20 @@ where
         // Determine system call capabilities.
         let mut caps = Capability::try_from((req, syscall_name))?;
 
+        // Check if system call is FD-only.
+        let is_fd = path_argv.iter().all(|arg| arg.path.is_none());
+
         // Check for chroot:
         //
-        // Delay Chdir to allow the common `cd /`. use case
-        // right after chroot.
+        // Delay Chdir to allow the common `cd /` use case right after chroot(2).
         let sandbox = request.get_sandbox();
         if sandbox.is_chroot() && !caps.contains(Capability::CAP_CHDIR) {
-            return Err(Errno::ENOENT);
+            return Err(if is_fd { Errno::EACCES } else { Errno::ENOENT });
         }
 
         // If sandboxing for all the selected capabilities is off, return immediately.
         let crypt = sandbox.enabled(Capability::CAP_CRYPT);
-        let hide = sandbox.enabled(Capability::CAP_STAT);
+        let hide = !is_fd && sandbox.enabled(Capability::CAP_STAT);
 
         // EXCEPTION: We do want to return success
         // to _access_(2) calls to magic paths in
@@ -398,22 +400,19 @@ where
                         // SAFETY: Allow `cd /` after chroot.
                         Ok(unsafe { request.continue_syscall() })
                     } else {
+                        // arg.path.is_some() -> is_fd = false: Return ENOENT.
                         Err(Errno::ENOENT)
                     };
                 }
 
                 paths[idx] = Some(path);
-            } else if let Some(arg_dirfd) = arg.dirfd {
-                #[allow(clippy::cast_possible_truncation)]
-                let dirfd = req.data.args[arg_dirfd] as RawFd;
+            } else if let Some(arg_idx) = arg.dirfd {
+                // Validate FD argument.
+                let dirfd = to_valid_fd(req.data.args[arg_idx])?;
 
                 if sandbox.is_chroot() {
-                    return if caps.contains(Capability::CAP_CHDIR) {
-                        // SAFETY: Do not allow fchdir after chroot.
-                        Err(Errno::EACCES)
-                    } else {
-                        Err(Errno::ENOENT)
-                    };
+                    // Return EACCES for FD-only system calls such as fchdir.
+                    return Err(if is_fd { Errno::EACCES } else { Errno::ENOENT });
                 }
 
                 if dirfd != libc::AT_FDCWD {
@@ -443,28 +442,18 @@ where
                         None
                     };
 
-                    let mut path = if let Some(path) = crypt_path {
+                    let path = if let Some(path) = crypt_path {
                         // SAFETY: Only regular files are encrypted.
                         CanonicalPath::new(path, FileType::Reg, arg.fsflags)?
                     } else {
                         CanonicalPath::new_fd(fd.into(), req.pid(), dirfd)?
                     };
 
-                    if arg.flags.contains(SysFlags::UNSAFE_CONT) {
-                        // FD not required if we're continuing...
-                        path.dir = None;
-                    }
-
                     paths[idx] = Some(path);
                 } else {
-                    let mut path =
+                    let path =
                         CanonicalPath::new_fd(libc::AT_FDCWD.into(), req.pid(), libc::AT_FDCWD)?;
 
-                    if arg.flags.contains(SysFlags::UNSAFE_CONT) {
-                        // FD not required if we're continuing...
-                        path.dir = None;
-                    }
-
                     paths[idx] = Some(path);
                 }
             } else {
