@@ -211,17 +211,6 @@ fn syscall_open_handler(
             return Err(Errno::EINVAL);
         }
 
-        // Quoting open(2):
-        //
-        // Currently, it is not possible to enable signal-driven I/O by
-        // specifying O_ASYNC when calling open(); use fcntl(2) to
-        // enable this flag.
-        //
-        // TODO: When this is possible, we must use fcntl to redirect signals
-        // to the sandbox thread making the actual open call or else we break
-        // async I/O.
-        // let o_async = flags.contains(OFlag::O_ASYNC);
-
         let req = request.scmpreq;
         let sandbox = request.get_sandbox();
 
@@ -330,7 +319,11 @@ fn syscall_open_handler(
         }
 
         // Validate file flags based on file type information.
-        let (is_blocking, may_crypt) = if let Some(ref file_type) = path.typ {
+        //
+        // is_dir: Used for mask to determine directory override.
+        // is_blocking: Used by interrupt handler to unblock FIFOs.
+        // may_crypt: Used by Crypt sandboxing.
+        let (is_dir, is_blocking, may_crypt) = if let Some(ref file_type) = path.typ {
             if o_creat && !o_tmpfl {
                 // Creating open with existing file:
                 //
@@ -348,7 +341,7 @@ fn syscall_open_handler(
                     // SAFETY:
                     // 1. We only ever attempt to encrypt regular files.
                     // 2. We do not support interruptions on regular file blocks.
-                    (false, true)
+                    (false, false, true)
                 }
                 FileType::Dir => {
                     // We know it's a directory, so let's assert it.
@@ -362,7 +355,7 @@ fn syscall_open_handler(
                     caps.remove(Capability::CAP_READ);
                     caps.insert(Capability::CAP_READDIR);
 
-                    (false, false)
+                    (true, false, false)
                 }
                 FileType::Blk | FileType::Unk => {
                     // SAFETY:
@@ -381,6 +374,7 @@ fn syscall_open_handler(
 
                     // SAFETY: This may or may not block, better safe than sorry.
                     (
+                        false,
                         !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
                         false,
                     )
@@ -388,6 +382,7 @@ fn syscall_open_handler(
                 FileType::Lnk => {
                     // SAFETY: This may or may not block, better safe than sorry.
                     (
+                        false,
                         !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
                         false,
                     )
@@ -395,6 +390,7 @@ fn syscall_open_handler(
                 FileType::Chr | FileType::Fifo | FileType::Sock => {
                     // SAFETY: character devices, fifos and sockets may block.
                     (
+                        false,
                         !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
                         false,
                     )
@@ -415,9 +411,10 @@ fn syscall_open_handler(
             //
             // flags.insert(OFlag::O_EXCL);
 
+            // Non-existing files are not directories.
             // Non-existing files do not block.
             // Non-existing files may be encrypted from scratch.
-            (false, true)
+            (false, false, true)
         };
 
         // SAFETY:
@@ -513,6 +510,16 @@ fn syscall_open_handler(
             // 3. Check for append-only path and edit flags argument.
             // We perform these check only if we're allowing.
             let crypted = if let Some(mask) = sandbox.is_masked(&path_check) {
+                let mask = if let Some(mask_dir) = &mask.mask_dir {
+                    // Override mask for directories as necessary.
+                    if is_dir {
+                        Some(mask_dir)
+                    } else {
+                        mask.mask_all.as_ref()
+                    }
+                } else {
+                    mask.mask_all.as_ref()
+                };
                 match mask {
                     None => path = CanonicalPath::new_null(),
                     Some(mask) => path = CanonicalPath::new_mask(mask, &path_check)?,
