@@ -36,14 +36,9 @@ pub(crate) fn sys_getxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::WANT_BASE,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "getxattr",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_getxattr_handler(request, &sandbox, path_args)
-        },
-    )
+    syscall_path_handler(request, "getxattr", argv, |path_args, request, sandbox| {
+        syscall_getxattr_handler(request, &sandbox, path_args)
+    })
 }
 
 pub(crate) fn sys_lgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -57,14 +52,9 @@ pub(crate) fn sys_lgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::NO_FOLLOW_LAST | FsFlags::WANT_BASE,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "lgetxattr",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_getxattr_handler(request, &sandbox, path_args)
-        },
-    )
+    syscall_path_handler(request, "lgetxattr", argv, |path_args, request, sandbox| {
+        syscall_getxattr_handler(request, &sandbox, path_args)
+    })
 }
 
 pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -75,83 +65,78 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "fgetxattr",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            // SAFETY:
-            // 1. SysArg has one element.
-            // 2. SysArg.path is None asserting dir is Some.
-            #[allow(clippy::disallowed_methods)]
-            let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
+    syscall_path_handler(request, "fgetxattr", argv, |path_args, request, sandbox| {
+        // SAFETY:
+        // 1. SysArg has one element.
+        // 2. SysArg.path is None asserting dir is Some.
+        #[allow(clippy::disallowed_methods)]
+        let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
-            let req = request.scmpreq;
-            let name = if req.data.args[1] != 0 {
-                const SIZ: usize = libc::PATH_MAX as usize;
-                let mut buf = Vec::new();
-                buf.try_reserve(SIZ).or(Err(Errno::ENOMEM))?;
-                buf.resize(SIZ, 0);
-                request.read_mem(&mut buf, req.data.args[1])?;
-                Some(buf)
-            } else {
-                None
-            };
-            let name = if let Some(ref name) = name {
-                CStr::from_bytes_until_nul(name)
-                    .or(Err(Errno::E2BIG))?
-                    .as_ptr()
-            } else {
-                std::ptr::null()
-            };
+        let req = request.scmpreq;
+        let name = if req.data.args[1] != 0 {
+            const SIZ: usize = libc::PATH_MAX as usize;
+            let mut buf = Vec::new();
+            buf.try_reserve(SIZ).or(Err(Errno::ENOMEM))?;
+            buf.resize(SIZ, 0);
+            request.read_mem(&mut buf, req.data.args[1])?;
+            Some(buf)
+        } else {
+            None
+        };
+        let name = if let Some(ref name) = name {
+            CStr::from_bytes_until_nul(name)
+                .or(Err(Errno::E2BIG))?
+                .as_ptr()
+        } else {
+            std::ptr::null()
+        };
 
-            if sandbox.locked_for(req.pid()) {
-                // SAFETY: Deny user.syd* extended attributes. name is either
-                // NULL or a valid nul-terminated C-String.
-                // SAFETY: Deny with ENODATA for stealth.
-                // SAFETY: Deny only if the Sandbox is locked for the process.
-                unsafe { denyxattr(name) }?;
-            }
+        if sandbox.locked_for(req.pid()) {
+            // SAFETY: Deny user.syd* extended attributes. name is either
+            // NULL or a valid nul-terminated C-String.
+            // SAFETY: Deny with ENODATA for stealth.
+            // SAFETY: Deny only if the Sandbox is locked for the process.
+            unsafe { denyxattr(name) }?;
+        }
 
-            // SAFETY: The size argument to the getxattr call
-            // must not be fully trusted, it can be overly large,
-            // and allocating a Vector of that capacity may overflow.
-            let len = usize::try_from(req.data.args[3]).or(Err(Errno::E2BIG))?;
-            let len = len.min(libc::PATH_MAX as usize); // Cap count at PATH_MAX
-            let mut buf = if len > 0 {
-                let mut buf = Vec::new();
-                buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-                buf.resize(len, 0);
-                Some(buf)
-            } else {
-                None
-            };
-            let ptr = match buf.as_mut() {
-                Some(b) => b.as_mut_ptr(),
-                None => std::ptr::null_mut(),
-            };
-            // SAFETY: In libc we trust.
-            let n = unsafe {
-                libc::fgetxattr(
-                    fd.as_raw_fd(),
-                    name,
-                    ptr as *mut _ as *mut libc::c_void,
-                    len,
-                )
-            };
-            #[allow(clippy::cast_sign_loss)]
-            let n = if n == -1 {
-                return Err(Errno::last());
-            } else {
-                n as usize
-            };
-            if let Some(buf) = buf {
-                request.write_mem(&buf[..n], req.data.args[2])?;
-            }
-            #[allow(clippy::cast_possible_wrap)]
-            Ok(request.return_syscall(n as i64))
-        },
-    )
+        // SAFETY: The size argument to the getxattr call
+        // must not be fully trusted, it can be overly large,
+        // and allocating a Vector of that capacity may overflow.
+        let len = usize::try_from(req.data.args[3]).or(Err(Errno::E2BIG))?;
+        let len = len.min(libc::PATH_MAX as usize); // Cap count at PATH_MAX
+        let mut buf = if len > 0 {
+            let mut buf = Vec::new();
+            buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
+            buf.resize(len, 0);
+            Some(buf)
+        } else {
+            None
+        };
+        let ptr = match buf.as_mut() {
+            Some(b) => b.as_mut_ptr(),
+            None => std::ptr::null_mut(),
+        };
+        // SAFETY: In libc we trust.
+        let n = unsafe {
+            libc::fgetxattr(
+                fd.as_raw_fd(),
+                name,
+                ptr as *mut _ as *mut libc::c_void,
+                len,
+            )
+        };
+        #[allow(clippy::cast_sign_loss)]
+        let n = if n == -1 {
+            return Err(Errno::last());
+        } else {
+            n as usize
+        };
+        if let Some(buf) = buf {
+            request.write_mem(&buf[..n], req.data.args[2])?;
+        }
+        #[allow(clippy::cast_possible_wrap)]
+        Ok(request.return_syscall(n as i64))
+    })
 }
 
 pub(crate) fn sys_getxattrat(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -192,9 +177,7 @@ pub(crate) fn sys_getxattrat(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "getxattrat",
         argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_getxattrat_handler(request, &sandbox, path_args)
-        },
+        |path_args, request, sandbox| syscall_getxattrat_handler(request, &sandbox, path_args),
     )
 }
 
@@ -206,14 +189,9 @@ pub(crate) fn sys_setxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "setxattr",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_setxattr_handler(request, &sandbox, path_args)
-        },
-    )
+    syscall_path_handler(request, "setxattr", argv, |path_args, request, sandbox| {
+        syscall_setxattr_handler(request, &sandbox, path_args)
+    })
 }
 
 pub(crate) fn sys_fsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -224,14 +202,9 @@ pub(crate) fn sys_fsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "fsetxattr",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_setxattr_handler(request, &sandbox, path_args)
-        },
-    )
+    syscall_path_handler(request, "fsetxattr", argv, |path_args, request, sandbox| {
+        syscall_setxattr_handler(request, &sandbox, path_args)
+    })
 }
 
 pub(crate) fn sys_lsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -242,86 +215,81 @@ pub(crate) fn sys_lsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::NO_FOLLOW_LAST | FsFlags::WANT_BASE,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "lsetxattr",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            // SAFETY: SysArg has one element.
-            #[allow(clippy::disallowed_methods)]
-            let path = path_args.0.as_ref().unwrap();
+    syscall_path_handler(request, "lsetxattr", argv, |path_args, request, sandbox| {
+        // SAFETY: SysArg has one element.
+        #[allow(clippy::disallowed_methods)]
+        let path = path_args.0.as_ref().unwrap();
 
-            let base = if path.base.is_empty() {
-                XPath::from_bytes(b".")
-            } else {
-                path.base
-            };
+        let base = if path.base.is_empty() {
+            XPath::from_bytes(b".")
+        } else {
+            path.base
+        };
 
-            let req = request.scmpreq;
+        let req = request.scmpreq;
 
-            let name = if req.data.args[1] != 0 {
-                const SIZ: usize = libc::PATH_MAX as usize;
-                let mut buf = Vec::new();
-                buf.try_reserve(SIZ).or(Err(Errno::ENOMEM))?;
-                buf.resize(SIZ, 0);
-                request.read_mem(&mut buf, req.data.args[1])?;
-                Some(buf)
-            } else {
-                None
-            };
-            let name = if let Some(ref name) = name {
-                CStr::from_bytes_until_nul(name)
-                    .or(Err(Errno::E2BIG))?
-                    .as_ptr()
-            } else {
-                std::ptr::null()
-            };
+        let name = if req.data.args[1] != 0 {
+            const SIZ: usize = libc::PATH_MAX as usize;
+            let mut buf = Vec::new();
+            buf.try_reserve(SIZ).or(Err(Errno::ENOMEM))?;
+            buf.resize(SIZ, 0);
+            request.read_mem(&mut buf, req.data.args[1])?;
+            Some(buf)
+        } else {
+            None
+        };
+        let name = if let Some(ref name) = name {
+            CStr::from_bytes_until_nul(name)
+                .or(Err(Errno::E2BIG))?
+                .as_ptr()
+        } else {
+            std::ptr::null()
+        };
 
-            if sandbox.locked_for(req.pid()) {
-                // SAFETY: Deny user.syd* extended attributes. name is either
-                // NULL or a valid nul-terminated C-String.
-                // SAFETY: Deny with ENODATA for stealth.
-                // SAFETY: Deny only if the Sandbox is locked for the process.
-                unsafe { denyxattr(name) }?;
+        if sandbox.locked_for(req.pid()) {
+            // SAFETY: Deny user.syd* extended attributes. name is either
+            // NULL or a valid nul-terminated C-String.
+            // SAFETY: Deny with ENODATA for stealth.
+            // SAFETY: Deny only if the Sandbox is locked for the process.
+            unsafe { denyxattr(name) }?;
+        }
+
+        // SAFETY: The size argument to the setxattr call
+        // must not be fully trusted, it can be overly large,
+        // and allocating a Vector of that capacity may overflow.
+        let (buf, len) = if req.data.args[3] == 0 {
+            (None, 0)
+        } else {
+            let len = usize::try_from(req.data.args[3]).or(Err(Errno::E2BIG))?;
+            let len = len.min(libc::PATH_MAX as usize); // Cap count at PATH_MAX.
+            let mut buf = Vec::new();
+            buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
+            buf.resize(len, 0);
+            request.read_mem(&mut buf, req.data.args[2])?;
+            (Some(buf), len)
+        };
+        let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
+
+        #[allow(clippy::cast_possible_truncation)]
+        let flags = req.data.args[4] as libc::c_int;
+
+        match &path.dir {
+            Some(fd) => {
+                // SAFETY: We use fchdir which is TOCTOU-free!
+                fchdir(fd)?;
             }
+            None => {
+                // SAFETY: `/` is never a symlink!
+                fchdir(ROOT_FILE())?;
+            }
+        };
 
-            // SAFETY: The size argument to the setxattr call
-            // must not be fully trusted, it can be overly large,
-            // and allocating a Vector of that capacity may overflow.
-            let (buf, len) = if req.data.args[3] == 0 {
-                (None, 0)
-            } else {
-                let len = usize::try_from(req.data.args[3]).or(Err(Errno::E2BIG))?;
-                let len = len.min(libc::PATH_MAX as usize); // Cap count at PATH_MAX.
-                let mut buf = Vec::new();
-                buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-                buf.resize(len, 0);
-                request.read_mem(&mut buf, req.data.args[2])?;
-                (Some(buf), len)
-            };
-            let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
-
-            #[allow(clippy::cast_possible_truncation)]
-            let flags = req.data.args[4] as libc::c_int;
-
-            match &path.dir {
-                Some(fd) => {
-                    // SAFETY: We use fchdir which is TOCTOU-free!
-                    fchdir(fd)?;
-                }
-                None => {
-                    // SAFETY: `/` is never a symlink!
-                    fchdir(ROOT_FILE())?;
-                }
-            };
-
-            // SAFETY: In libc we trust.
-            let res = base.with_nix_path(|cstr| unsafe {
-                libc::lsetxattr(cstr.as_ptr(), name, buf, len, flags)
-            })?;
-            Errno::result(res).map(|_| request.return_syscall(0))
-        },
-    )
+        // SAFETY: In libc we trust.
+        let res = base.with_nix_path(|cstr| unsafe {
+            libc::lsetxattr(cstr.as_ptr(), name, buf, len, flags)
+        })?;
+        Errno::result(res).map(|_| request.return_syscall(0))
+    })
 }
 
 pub(crate) fn sys_setxattrat(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -362,9 +330,7 @@ pub(crate) fn sys_setxattrat(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "setxattrat",
         argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_setxattrat_handler(request, &sandbox, path_args)
-        },
+        |path_args, request, sandbox| syscall_setxattrat_handler(request, &sandbox, path_args),
     )
 }
 
@@ -380,7 +346,7 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "flistxattr",
         argv,
-        |path_args: PathArgs, request, sandbox| {
+        |path_args, request, sandbox| {
             // SAFETY:
             // 1. SysArg has one element.
             // 2. SysArg.path is None asserting dir is Some.
@@ -446,14 +412,9 @@ pub(crate) fn sys_listxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::WANT_BASE,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "listxattr",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_listxattr_handler(request, &sandbox, path_args)
-        },
-    )
+    syscall_path_handler(request, "listxattr", argv, |path_args, request, sandbox| {
+        syscall_listxattr_handler(request, &sandbox, path_args)
+    })
 }
 
 pub(crate) fn sys_llistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -471,9 +432,7 @@ pub(crate) fn sys_llistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "llistxattr",
         argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_listxattr_handler(request, &sandbox, path_args)
-        },
+        |path_args, request, sandbox| syscall_listxattr_handler(request, &sandbox, path_args),
     )
 }
 
@@ -489,9 +448,7 @@ pub(crate) fn sys_removexattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "removexattr",
         argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_removexattr_handler(request, &sandbox, path_args)
-        },
+        |path_args, request, sandbox| syscall_removexattr_handler(request, &sandbox, path_args),
     )
 }
 
@@ -534,9 +491,7 @@ pub(crate) fn sys_listxattrat(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "listxattrat",
         argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_listxattrat_handler(request, &sandbox, path_args)
-        },
+        |path_args, request, sandbox| syscall_listxattrat_handler(request, &sandbox, path_args),
     )
 }
 
@@ -552,9 +507,7 @@ pub(crate) fn sys_fremovexattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "fremovexattr",
         argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_removexattr_handler(request, &sandbox, path_args)
-        },
+        |path_args, request, sandbox| syscall_removexattr_handler(request, &sandbox, path_args),
     )
 }
 
@@ -570,7 +523,7 @@ pub(crate) fn sys_lremovexattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "lremovexattr",
         argv,
-        |path_args: PathArgs, request, sandbox| {
+        |path_args, request, sandbox| {
             // SAFETY: SysArg has one element.
             #[allow(clippy::disallowed_methods)]
             let path = path_args.0.as_ref().unwrap();
@@ -666,9 +619,7 @@ pub(crate) fn sys_removexattrat(request: UNotifyEventRequest) -> ScmpNotifResp {
         request,
         "removexattrat",
         argv,
-        |path_args: PathArgs, request, sandbox| {
-            syscall_removexattrat_handler(request, &sandbox, path_args)
-        },
+        |path_args, request, sandbox| syscall_removexattrat_handler(request, &sandbox, path_args),
     )
 }
 
