@@ -227,6 +227,7 @@ use nix::{
     mount::MsFlags,
     sched::{unshare, CloneFlags},
     sys::{
+        resource::{getrlimit, Resource},
         signal::{sigaction, signal, SaFlags, SigAction, SigHandler, SigSet, Signal},
         socket::{socket, AddressFamily, SockFlag, SockType},
         stat::Mode,
@@ -245,6 +246,7 @@ use crate::{
     hash::{aes_ctr_info, hmac_sha256_info},
     landlock::{RulesetStatus, ABI},
     path::{mask_path, XPath, XPathBuf},
+    proc::{proc_fs_file_max, proc_fs_nr_open},
     sealbox::check_mseal_support,
     spec::{speculation_get, SpeculationFeature},
 };
@@ -745,6 +747,23 @@ pub fn syd_info() {
     };
     println!("LSMs: {lsms}.");
 
+    // Print information on open file limits.
+    let (nofile_soft, nofile_hard) = getrlimit(Resource::RLIMIT_NOFILE).unwrap_or((0, 0));
+    print!("Open file limits: {nofile_soft} soft, {nofile_hard} hard, ");
+
+    // Lookup system-wide open file limits.
+    match crate::config::proc_init() {
+        Ok(_) => {
+            let file_max = proc_fs_file_max().unwrap_or(0);
+            let nr_open = proc_fs_nr_open().unwrap_or(0);
+            crate::config::proc_close();
+            println!("{nr_open} nr_open, {file_max} file-max");
+        }
+        Err(errno) => {
+            println!("? nr_open, ? file-max (error: {errno})");
+        }
+    }
+
     let uname = match uname() {
         Ok(info) => OsStr::to_str(info.release()).unwrap_or("?").to_string(),
         Err(_) => "?".to_string(),
@@ -1046,16 +1065,21 @@ pub fn ignore_signals() -> Result<(), Errno> {
     // Iterate through all possible signals and set them to be ignored.
     // Step 1: Normal signals.
     for signal in Signal::iterator() {
-        if !matches!(
-            signal,
-            Signal::SIGALRM | Signal::SIGKILL | Signal::SIGSTOP | Signal::SIGPIPE | Signal::SIGCHLD
-        ) && !is_coredump(signal as i32)
-        {
-            // 1. Can not ignore SIGKILL and SIGSTOP.
-            // 2. Do not need to ignore Signals with default action Core.
-            // 3. Ignoring SIGCHLD changes wait semantics which we cannot do.
-            // 4. SIGPIPE must be handled outside this function.
-            ignore_signal(signal)?;
+        // 1. Can not ignore SIGKILL and SIGSTOP.
+        // 2. Do not need to ignore Signals with default action Core.
+        // 3. Ignoring SIGCHLD changes wait semantics which we cannot do.
+        // 4. SIGPIPE must be handled outside this function.
+        // 5. SIGPROF is used for profiling.
+        match signal {
+            Signal::SIGALRM
+            | Signal::SIGKILL
+            | Signal::SIGSTOP
+            | Signal::SIGPIPE
+            | Signal::SIGCHLD => {}
+            #[cfg(feature = "prof")]
+            Signal::SIGPROF => {}
+            signal if is_coredump(signal as i32) => {}
+            signal => ignore_signal(signal)?,
         }
     }
 
@@ -1702,13 +1726,10 @@ pub fn human_size(bytes: usize) -> String {
     format!("{:.2}{}", size, SIZES[i])
 }
 
-#[cfg(target_env = "musl")]
-pub(crate) type IoctlRequest = libc::c_int;
-#[cfg(not(target_env = "musl"))]
-pub(crate) type IoctlRequest = libc::c_ulong;
-
-const SIOCGIFFLAGS: IoctlRequest = libc::SIOCGIFFLAGS as IoctlRequest;
-const SIOCSIFFLAGS: IoctlRequest = libc::SIOCSIFFLAGS as IoctlRequest;
+#[allow(clippy::unnecessary_cast)]
+const SIOCGIFFLAGS: u64 = libc::SIOCGIFFLAGS as u64;
+#[allow(clippy::unnecessary_cast)]
+const SIOCSIFFLAGS: u64 = libc::SIOCSIFFLAGS as u64;
 
 /// Functionally equivalent to "ifconfig lo up".
 pub fn bring_up_loopback() -> Result<(), Errno> {
@@ -1741,28 +1762,38 @@ pub fn bring_up_loopback() -> Result<(), Errno> {
             0,
             0,
         ],
-        // SAFETY: Manually initialize ifr_ifru
+        // SAFETY: Manually initialize ifr_ifru.
         ifr_ifru: unsafe { std::mem::zeroed() },
     };
 
-    // SAFETY: Get the current flags
-    if unsafe { libc::ioctl(sock.as_raw_fd(), SIOCGIFFLAGS, &mut ifreq) } != 0 {
-        return Err(Errno::last());
-    }
+    // SAFETY: Get the current flags.
+    Errno::result(unsafe {
+        libc::syscall(
+            libc::SYS_ioctl,
+            sock.as_raw_fd(),
+            SIOCGIFFLAGS as libc::c_ulong,
+            &mut ifreq,
+        )
+    })?;
 
-    // Modify the flags to bring up the interface
+    // Modify the flags to bring up the interface.
+    //
     // SAFETY: We're accessing the field of a union here.
     #[allow(clippy::cast_possible_truncation)]
     unsafe {
         ifreq.ifr_ifru.ifru_flags |= (libc::IFF_UP | libc::IFF_RUNNING) as libc::c_short
     };
 
-    // SAFETY: Set the new flags
-    if unsafe { libc::ioctl(sock.as_raw_fd(), SIOCSIFFLAGS, &mut ifreq) } == 0 {
-        Ok(())
-    } else {
-        Err(Errno::last())
-    }
+    // SAFETY: Set the new flags.
+    Errno::result(unsafe {
+        libc::syscall(
+            libc::SYS_ioctl,
+            sock.as_raw_fd(),
+            SIOCSIFFLAGS as libc::c_ulong,
+            &mut ifreq,
+        )
+    })
+    .map(drop)
 }
 
 const DEV_NULL_MAJOR: u32 = 1;
