@@ -23,7 +23,7 @@ use crate::{
     config::*,
     fs::{lock_fd, retry_on_eintr, unlock_fd},
     proc::*,
-    syslog::{LogLevel, *},
+    syslog::LogLevel,
 };
 
 // Whether we concluded the output is a TTY.
@@ -242,6 +242,8 @@ impl Write for LockedWriter<'_> {
 pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Result<(), Errno> {
     use std::os::unix::ffi::OsStrExt;
 
+    use crate::syslog::{init_global_syslog, parse_loglevel};
+
     // Parse the desired console log level from ENV_LOG,
     // or use default_level if not set/invalid.
     let level = if let Some(val) = std::env::var_os(ENV_LOG) {
@@ -312,6 +314,8 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
 pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
     use std::os::unix::ffi::OsStrExt;
 
+    use crate::syslog::{global_syslog, init_global_syslog, parse_loglevel};
+
     // Parse the desired console log level from ENV_LOG,
     // or use default_level if not set/invalid.
     let level = if let Some(val) = std::env::var_os(ENV_LOG) {
@@ -382,7 +386,7 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
     // We remove and readd to reorder for better visibility.
     if let Some(pid_v) = msg.remove("pid").and_then(|v| v.as_i64()) {
         #[allow(clippy::cast_possible_truncation)]
-        let pid = Pid::from_raw(pid_v as nix::libc::pid_t);
+        let pid = Pid::from_raw(pid_v as libc::pid_t);
         if pid.as_raw() != 0 {
             if add_context {
                 if let Ok(cmd) = proc_cmdline(pid) {
@@ -511,7 +515,8 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
     // If there's a "pid", we might add cmd/cwd, etc.
     // We remove and readd to reorder for better visibility.
     if let Some(pid_v) = msg.remove("pid").and_then(|v| v.as_i64()) {
-        let pid = Pid::from_raw(pid_v as nix::libc::pid_t);
+        #[allow(clippy::cast_possible_truncation)]
+        let pid = Pid::from_raw(pid_v as libc::pid_t);
         if pid.as_raw() != 0 {
             if add_context {
                 if let Ok(cmd) = proc_cmdline(pid) {
@@ -568,13 +573,6 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         format!("{{\"ctx\":\"log\",\"op\":\"serialize\",\"error\": \"{e}\"}}")
     });
 
-    // Warn or higher goes to syslog too unless SYD_NO_SYSLOG is set.
-    if std::env::var_os(ENV_NO_SYSLOG).is_none()
-        && level.as_u8() <= crate::syslog::LogLevel::Warn.as_u8()
-    {
-        host_syslog(&msg_data);
-    }
-
     let msg_info = if tty {
         // A "pretty" output for TTY.
         Some(serde_json::to_string_pretty(&msg).unwrap_or_else(|e| {
@@ -617,14 +615,14 @@ pub fn now() -> u64 {
 #[allow(clippy::arithmetic_side_effects)]
 #[allow(deprecated)]
 fn format_iso8601(timestamp: u64) -> String {
-    let timestamp = match nix::libc::time_t::try_from(timestamp) {
+    let timestamp = match libc::time_t::try_from(timestamp) {
         Ok(t) => t,
         Err(_) => return "error on time()".to_string(),
     };
     let mut tm = std::mem::MaybeUninit::uninit();
 
     // SAFETY: gmtime_r returns NULL on error.
-    if unsafe { nix::libc::gmtime_r(&timestamp, tm.as_mut_ptr()) }.is_null() {
+    if unsafe { libc::gmtime_r(&timestamp, tm.as_mut_ptr()) }.is_null() {
         return "error on gmtime_r()".to_string();
     }
 
