@@ -45,9 +45,10 @@ use crate::{
     config::*,
     err::{err2no, SydResult},
     error,
+    fs::{retry_on_eintr, seccomp_export_pfc},
     hook::HandlerMap,
     info,
-    sandbox::{Capability, Sandbox},
+    sandbox::{Capability, Flags, Sandbox},
     workers::{
         aes::{AesMap, AesWorker},
         emu::Worker,
@@ -84,8 +85,7 @@ extern "C" fn handle_sigalrm(_: libc::c_int) {}
 pub(crate) struct ThreadPool {
     core_size: usize,
     keep_alive: u16,
-    safe_setid: bool,
-    scmp_ssb: bool,
+    flags: Flags,
     fd: RawFd,
     pub(crate) epoll: Arc<Epoll>,
     pub(crate) cache: Arc<WorkerCache<'static>>,
@@ -114,8 +114,7 @@ impl ThreadPool {
     pub(crate) fn new(
         epoll: Epoll,
         fd: RawFd,
-        safe_setid: bool,
-        scmp_ssb: bool,
+        flags: Flags,
         core_size: usize,
         keep_alive: u16,
         sandbox: Arc<RwLock<Sandbox>>,
@@ -141,9 +140,8 @@ impl ThreadPool {
             handlers,
             core_size,
             keep_alive,
-            safe_setid,
-            scmp_ssb,
             epoll,
+            flags,
             should_exit: Arc::new(AtomicBool::new(false)),
             worker_data: Arc::new(WorkerData::default()),
             bind_map: Arc::new(RwLock::new(HashMap::default())),
@@ -160,12 +158,7 @@ impl ThreadPool {
         match ExportMode::from_env() {
             Some(ExportMode::BerkeleyPacketFilter) => {
                 // Worker rules
-                let ctx = Worker::prepare_confine(
-                    self.fd,
-                    self.epoll.0.as_raw_fd(),
-                    self.safe_setid,
-                    self.scmp_ssb,
-                )?;
+                let ctx = Worker::prepare_confine(self.fd, self.epoll.0.as_raw_fd(), self.flags)?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -175,13 +168,7 @@ impl ThreadPool {
 
                 // Interrupter rules
                 // We pass dry_run=true to avoid Landlock confinement.
-                let ctx = Interrupter::prepare_confine(
-                    self.fd,
-                    getpid(),
-                    self.safe_setid,
-                    self.scmp_ssb,
-                    true,
-                )?;
+                let ctx = Interrupter::prepare_confine(self.fd, getpid(), self.flags, true)?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -190,9 +177,9 @@ impl ThreadPool {
                 ctx.export_bpf(file)?;
 
                 // IPC thread rules
-                // We pass dummy RawFd=0 for epoll FD.
+                // We pass dummy RawFd=2525 for epoll FD.
                 // We pass dry_run=true to avoid Landlock confinement.
-                let ctx = IpcWorker::prepare_confine(0, self.safe_setid, self.scmp_ssb, true)?;
+                let ctx = IpcWorker::prepare_confine(2525, self.flags, true)?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -201,7 +188,7 @@ impl ThreadPool {
                 ctx.export_bpf(file)?;
 
                 // Aes worker rules
-                let ctx = AesWorker::prepare_confine(self.safe_setid, self.scmp_ssb)?;
+                let ctx = AesWorker::prepare_confine(self.flags)?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -210,32 +197,24 @@ impl ThreadPool {
                 ctx.export_bpf(file)?;
             }
             Some(ExportMode::PseudoFiltercode) => {
-                println!("# Syd monitor rules");
-                let ctx = Worker::prepare_confine(
-                    self.fd,
-                    self.epoll.0.as_raw_fd(),
-                    self.safe_setid,
-                    self.scmp_ssb,
-                )?;
-                ctx.export_pfc(std::io::stdout())?;
+                println!("# Syd monitor rules with seccomp fd {}", self.fd);
+                let ctx = Worker::prepare_confine(self.fd, self.epoll.0.as_raw_fd(), self.flags)?;
+                print!("{}", seccomp_export_pfc(&ctx)?);
 
-                println!("# Syd interrupter rules");
-                let ctx = Interrupter::prepare_confine(
-                    self.fd,
-                    getpid(),
-                    self.safe_setid,
-                    self.scmp_ssb,
-                    false,
-                )?;
-                ctx.export_pfc(std::io::stdout())?;
+                // We pass dry_run=true to avoid Landlock confinement.
+                println!("# Syd interrupter rules with seccomp fd {}", self.fd);
+                let ctx = Interrupter::prepare_confine(self.fd, getpid(), self.flags, true)?;
+                print!("{}", seccomp_export_pfc(&ctx)?);
 
+                // We pass dummy RawFd=2525 for epoll FD.
+                // We pass dry_run=true to avoid Landlock confinement.
                 println!("# Syd ipc rules");
-                let ctx = IpcWorker::prepare_confine(0, self.safe_setid, self.scmp_ssb, true)?;
-                ctx.export_pfc(std::io::stdout())?;
+                let ctx = IpcWorker::prepare_confine(2525, self.flags, true)?;
+                print!("{}", seccomp_export_pfc(&ctx)?);
 
                 println!("# Syd encryptor rules");
-                let ctx = AesWorker::prepare_confine(self.safe_setid, self.scmp_ssb)?;
-                ctx.export_pfc(std::io::stdout())?;
+                let ctx = AesWorker::prepare_confine(self.flags)?;
+                print!("{}", seccomp_export_pfc(&ctx)?);
             }
             _ => {}
         }
@@ -333,14 +312,15 @@ impl ThreadPool {
                 // spawned threads don't need to reapply the same filter
                 // as it is inherited.
                 let dry_run = env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
+                let safe_setid = self.flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
                 #[allow(clippy::disallowed_methods)]
                 let mut confined = if Sandbox::locked_once() {
-                    let ctx = Worker::prepare_confine(self.fd, self.epoll.0.as_raw_fd(), self.safe_setid, self.scmp_ssb)
+                    let ctx = Worker::prepare_confine(self.fd, self.epoll.0.as_raw_fd(), self.flags)
                         .expect("prepare monitor thread confinement");
                     if !dry_run {
                         info!("ctx": "confine", "op": "confine_monitor_thread",
                             "msg": format!("monitor thread confined with{} SROP mitigation",
-                                if self.safe_setid { "out" } else { "" }));
+                                if safe_setid { "out" } else { "" }));
                         ctx.load().expect("confine monitor thread");
                     } else {
                         error!("ctx": "confine", "op": "confine_monitor_thread",
@@ -376,12 +356,12 @@ impl ThreadPool {
                     // Confine as necessary.
                     #[allow(clippy::disallowed_methods)]
                     if !confined && Sandbox::locked_once() {
-                        let ctx = Worker::prepare_confine(self.fd, self.epoll.0.as_raw_fd(), self.safe_setid, self.scmp_ssb)
+                        let ctx = Worker::prepare_confine(self.fd, self.epoll.0.as_raw_fd(), self.flags)
                             .expect("prepare monitor thread confinement");
                         if !dry_run {
                             info!("ctx": "confine", "op": "confine_monitor_thread",
                                 "msg": format!("monitor thread confined with{} SROP mitigation",
-                                    if self.safe_setid { "out" } else { "" }));
+                                    if safe_setid { "out" } else { "" }));
                             ctx.load().expect("confine monitor thread");
                         } else {
                             error!("ctx": "confine", "op": "confine_monitor_thread",
@@ -433,14 +413,15 @@ impl ThreadPool {
         // SAFETY: Register the handler for SIGALRM.
         unsafe { sigaction(Signal::SIGALRM, &sig_action) }?;
 
-        Interrupter::new(
-            self.fd,
-            self.safe_setid,
-            self.scmp_ssb,
-            Arc::clone(&self.should_exit),
-            Arc::clone(&self.cache),
-        )
-        .try_spawn()
+        retry_on_eintr(|| {
+            Interrupter::new(
+                self.fd,
+                self.flags,
+                Arc::clone(&self.should_exit),
+                Arc::clone(&self.cache),
+            )
+            .try_spawn()
+        })
     }
 
     /// Try to create a new encryption thread.
@@ -450,7 +431,12 @@ impl ThreadPool {
         files: AesMap,
         memfd: bool,
     ) -> Result<JoinHandle<()>, Errno> {
-        AesWorker::new(fdalg, files, memfd, self.safe_setid, self.scmp_ssb).try_spawn()
+        let worker = AesWorker::new(fdalg, files, self.flags, memfd);
+
+        // AesWorker has only RawFds as Fds which
+        // we do _not_ want to duplicate on clone,
+        // so we can get away with a clone here...
+        retry_on_eintr(|| worker.clone().try_spawn())
     }
 
     /// Try to create a new worker thread as needed.
@@ -506,19 +492,21 @@ impl ThreadPool {
 
         // Try to spawn a new worker.
         Ok(Some((
-            Worker::new(
-                self.fd,
-                Arc::clone(&self.epoll),
-                Arc::clone(&self.cache),
-                Arc::clone(&self.sandbox),
-                Arc::clone(&self.handlers),
-                keep_alive,
-                Arc::clone(&self.should_exit),
-                Arc::clone(&self.worker_data),
-                Arc::clone(&self.bind_map),
-                self.crypt_map.as_ref().map(Arc::clone),
-            )
-            .try_spawn(confined)?,
+            retry_on_eintr(|| {
+                Worker::new(
+                    self.fd,
+                    Arc::clone(&self.epoll),
+                    Arc::clone(&self.cache),
+                    Arc::clone(&self.sandbox),
+                    Arc::clone(&self.handlers),
+                    keep_alive,
+                    Arc::clone(&self.should_exit),
+                    Arc::clone(&self.worker_data),
+                    Arc::clone(&self.bind_map),
+                    self.crypt_map.as_ref().map(Arc::clone),
+                )
+                .try_spawn(confined)
+            })?,
             keep_alive.is_none(),
         )))
     }
