@@ -28,7 +28,10 @@ use std::{
     fs::File,
     io::{BufReader, IoSlice, IoSliceMut, Read},
     ops::Range,
-    os::{fd::RawFd, unix::ffi::OsStrExt},
+    os::{
+        fd::{AsFd, OwnedFd, RawFd},
+        unix::ffi::OsStrExt,
+    },
 };
 
 use btoi::{btoi, btoi_radix};
@@ -118,8 +121,8 @@ pub struct Status {
     pub sig_caught: SydSigSet,
 }
 
-// Safe `OFlags` to use when opening `/proc` files.
-const OFLAG_PROC: OFlag =
+/// Safe `OFlags` to use when opening `/proc` files.
+pub const OFLAG_PROC: OFlag =
     OFlag::from_bits_retain(libc::O_RDONLY | libc::O_CLOEXEC | libc::O_NOFOLLOW | libc::O_NOCTTY);
 
 // major 204 is a mess -- "Low-density serial ports"
@@ -223,16 +226,29 @@ pub fn proc_statm(pid: Pid) -> Result<Statm, Errno> {
 }
 
 /// Return the status of the given process.
+#[inline]
 pub fn proc_status(pid: Pid) -> Result<Status, Errno> {
+    proc_status_read(proc_status_open(pid)?)
+}
+
+/// Open a FD to /proc/$pid/status and return an OwnedFd.
+pub fn proc_status_open(pid: Pid) -> Result<OwnedFd, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"status");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
-        .map(File::from)
-        .or(Err(Errno::ESRCH))?;
+    safe_open(PROC_FILE(), &path, OFLAG_PROC).or(Err(Errno::ESRCH))
+}
 
-    let mut buf = [0; 2048]; // A typical status file is about 1000 bytes.
-    map_result(parse_status(read_to_end(&mut file, &mut buf)?))
+/// Return the status of the given process from the given Fd.
+pub fn proc_status_read<Fd: AsFd>(fd: Fd) -> Result<Status, Errno> {
+    // A typical status file is about 1500 bytes.
+    //
+    // SAFETY: read_to_end will return EOVERFLOW if
+    // proc_pid_status(5) ever grows beyond 2048 bytes
+    // so we will know.
+    let mut buf = [0; 2048];
+
+    map_result(parse_status(read_to_end(fd, &mut buf)?))
 }
 
 /// Retrieves the thread group ID (TGID) for the specified thread ID (TID).
@@ -485,14 +501,23 @@ pub fn proc_mmap(pid: Pid, addr: u64) -> Result<SydMemoryMap, Errno> {
     Err(Errno::EFAULT)
 }
 
-/// Retrieve the system pipe max limit.
+/// Return the system pipe max limit.
+#[inline]
 pub fn proc_pipemax() -> Result<libc::c_int, Errno> {
-    let fd = safe_open(
+    proc_pipemax_read(proc_pipemax_open()?)
+}
+
+/// Open a FD to /proc/sys/fs/pipe-max-size and return an OwnedFd.
+pub fn proc_pipemax_open() -> Result<OwnedFd, Errno> {
+    safe_open(
         PROC_FILE(),
         XPath::from_bytes(b"sys/fs/pipe-max-size"),
         OFLAG_PROC,
-    )?;
+    )
+}
 
+/// Retrieve the system pipe max limit from the given Fd.
+pub fn proc_pipemax_read<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
     // Read up to 24 bytes.
     let mut data = [0u8; 24];
     let mut nread = 0;
@@ -506,7 +531,7 @@ pub fn proc_pipemax() -> Result<libc::c_int, Errno> {
         }
     }
 
-    btoi::<libc::c_int>(&data[..nread]).or(Err(Errno::EINVAL))
+    btoi::<libc::c_int>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
 }
 
 /// Retrieves the value of /proc/sys/fs/file-max.
@@ -533,6 +558,30 @@ pub fn proc_fs_file_max() -> Result<u64, Errno> {
     btoi::<u64>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
 }
 
+/// Retrieves the value of /proc/sys/fs/nr_open.
+pub fn proc_fs_nr_open() -> Result<u64, Errno> {
+    let fd = safe_open(
+        PROC_FILE(),
+        XPath::from_bytes(b"sys/fs/nr_open"),
+        OFLAG_PROC,
+    )?;
+
+    // Read up to 24 bytes + 1 bytes for \n.
+    let mut data = [0u8; 25];
+    let mut nread = 0;
+    while nread < data.len() {
+        #[allow(clippy::arithmetic_side_effects)]
+        match read(&fd, &mut data[nread..]) {
+            Ok(0) => break,
+            Ok(n) => nread += n,
+            Err(Errno::EINTR) => continue,
+            Err(_) => return Err(Errno::ESRCH),
+        }
+    }
+
+    btoi::<u64>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
+}
+
 /// Retrieves the value of /proc/sys/vm/mmap_min_addr.
 #[allow(clippy::disallowed_methods)]
 pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
@@ -1234,7 +1283,7 @@ pub fn proc_max_open_files(pid: Pid) -> Result<LimitValue, Errno> {
 /// returns `Err(Errno::EBADF)`. Other errors (permissions, invalid pid, etc.)
 /// are returned immediately.
 pub fn proc_rand_fd(pid: Pid) -> Result<RawFd, Errno> {
-    const KCMP_FILE: u64 = 0;
+    const KCMP_FILE: libc::c_long = 0;
 
     // SAFETY: Cap to a sane maximum because sufficiently
     // big values of the hard limit tend to return ENOMEM.
@@ -1261,14 +1310,17 @@ pub fn proc_rand_fd(pid: Pid) -> Result<RawFd, Errno> {
         // be implemented in the host kernel.
         // SAFETY: There's no libc wrapper for kcmp.
         #[allow(unsafe_code)]
+        #[allow(clippy::cast_lossless)]
+        #[allow(clippy::cast_possible_wrap)]
+        #[allow(clippy::cast_sign_loss)]
         let result = Errno::result(unsafe {
             libc::syscall(
                 libc::SYS_kcmp,
-                pid.as_raw(),
-                pid.as_raw(),
+                pid.as_raw() as libc::c_long,
+                pid.as_raw() as libc::c_long,
                 KCMP_FILE,
-                fd_rand,
-                fd_rand,
+                fd_rand as libc::c_ulong as libc::c_long,
+                fd_rand as libc::c_ulong as libc::c_long,
             )
         })
         .map(drop);
