@@ -1268,6 +1268,20 @@ impl<K, V> SealableHashMap<K, V> {
             SealableHashMap::Sealed(s) => s.len(),
         }
     }
+
+    /// Retain only the entries for which `f(key, value)` returns true.
+    pub fn retain<F>(&mut self, f: F) -> Result<(), Errno>
+    where
+        F: FnMut(&K, &mut V) -> bool,
+    {
+        match self {
+            SealableHashMap::Unsealed(m) => {
+                m.retain(f);
+                Ok(())
+            }
+            SealableHashMap::Sealed(_) => Err(Errno::EPERM),
+        }
+    }
 }
 
 impl<K, V> Default for SealableHashMap<K, V> {
@@ -1473,6 +1487,20 @@ impl<K, V> SealableAHashMap<K, V> {
             SealableAHashMap::Sealed(s) => s.len(),
         }
     }
+
+    /// Retain only the entries for which `f(key, value)` returns true.
+    pub fn retain<F>(&mut self, f: F) -> Result<(), Errno>
+    where
+        F: FnMut(&K, &mut V) -> bool,
+    {
+        match self {
+            SealableAHashMap::Unsealed(m) => {
+                m.retain(f);
+                Ok(())
+            }
+            SealableAHashMap::Sealed(_) => Err(Errno::EPERM),
+        }
+    }
 }
 
 impl<K, V> Default for SealableAHashMap<K, V> {
@@ -2138,4 +2166,123 @@ mod tests {
         let m3 = SealableAHashMap::new(h3);
         assert_ne!(m3, sealed1);
     }
+
+    #[test]
+    fn test_hashmap_retain_removes_odds() {
+        let mut shm = SealableHashMap::new(HashMap::from([(1, 10), (2, 20), (3, 30), (4, 40)]));
+        // Keep only even keys
+        shm.retain(|k, _v| *k % 2 == 0).unwrap();
+        let collected: Vec<_> = shm.into_iter().map(|(&k, &v)| (k, v)).collect();
+        assert_eq!(collected.len(), 2);
+        assert!(collected.contains(&(2, 20)));
+        assert!(collected.contains(&(4, 40)));
+    }
+
+    #[test]
+    fn test_hashmap_retain_mutates_values() {
+        let mut shm = SealableHashMap::new(HashMap::from([(1, 1), (2, 2), (3, 3)]));
+        // Double the values for keys >= 2 and keep those entries
+        shm.retain(|k, v| {
+            *v *= 2;
+            *k >= 2
+        })
+        .unwrap();
+        // After retain, only keys 2 and 3 should remain, with doubled values
+        assert_eq!(shm.len(), 2);
+        assert_eq!(shm.get(&2), Some(&4));
+        assert_eq!(shm.get(&3), Some(&6));
+    }
+
+    #[test]
+    fn test_hashmap_retain_all() {
+        let mut shm = SealableHashMap::new(HashMap::from([('a', 100), ('b', 200)]));
+        // Retain everything
+        shm.retain(|_, _| true).unwrap();
+        assert_eq!(shm.len(), 2);
+        assert_eq!(shm.get(&'a'), Some(&100));
+        assert_eq!(shm.get(&'b'), Some(&200));
+    }
+
+    #[test]
+    fn test_hashmap_retain_none() {
+        let mut shm = SealableHashMap::new(HashMap::from([("x", 9), ("y", 8)]));
+        // Retain nothing
+        shm.retain(|_, _| false).unwrap();
+        assert!(shm.is_empty());
+    }
+
+    #[test]
+    fn test_hashmap_retain_on_empty_map() {
+        let mut shm: SealableHashMap<i32, i32> = SealableHashMap::default();
+        // Should be a no-op but still Ok
+        shm.retain(|_, _| unreachable!()).unwrap();
+        assert!(shm.is_empty());
+    }
+
+    #[test]
+    fn test_hashmap_sealed_retain_error() {
+        let shm = SealableHashMap::new(HashMap::from([(1, 1)]));
+        let mut sealed = shm.seal().unwrap();
+        // Attempting to retain on sealed map should return EPERM
+        let err = sealed.retain(|_, _| true).unwrap_err();
+        assert_eq!(err, Errno::EPERM);
+    }
+
+    // Tests for SealableAHashMap::retain
+    #[test]
+    fn test_ahashmap_retain_removes_small_values() {
+        let mut sahm = SealableAHashMap::new(AHashMap::from([(10, 1), (20, 2), (30, 3), (40, 4)]));
+        // Keep only entries with value > 2
+        sahm.retain(|_, v| *v > 2).unwrap();
+        let mut collected: Vec<_> = sahm.into_iter().map(|(&k, &v)| (k, v)).collect();
+        collected.sort_unstable();
+        assert_eq!(collected, vec![(30, 3), (40, 4)]);
+    }
+
+    #[ignore]
+    #[test]
+    fn test_ahashmap_retain_mutates_keys_and_values() {
+        // Note: mutation applies only to values
+        let mut sahm = SealableAHashMap::new(AHashMap::from([
+            ("one".to_string(), 1),
+            ("two".to_string(), 2),
+            ("three".to_string(), 3),
+        ]));
+        // For entries with even length keys, add 10 to the value and keep
+        sahm.retain(|k, v| {
+            if k.len() % 2 == 0 {
+                *v += 10;
+                true
+            } else {
+                false
+            }
+        })
+        .unwrap();
+        assert_eq!(sahm.len(), 1);
+        assert_eq!(sahm.get("two"), Some(&12));
+    }
+
+    #[test]
+    fn test_ahashmap_retain_all_and_none() {
+        let mut sahm = SealableAHashMap::new(AHashMap::from([(1, 100)]));
+        sahm.retain(|_, _| true).unwrap();
+        assert_eq!(sahm.len(), 1);
+        sahm.retain(|_, _| false).unwrap();
+        assert!(sahm.is_empty());
+    }
+
+    #[test]
+    fn test_ahashmap_retain_on_empty_map() {
+        let mut sahm: SealableAHashMap<u8, u8> = SealableAHashMap::default();
+        sahm.retain(|_, _| unreachable!()).unwrap();
+        assert!(sahm.is_empty());
+    }
+
+    #[test]
+    fn test_ahashmap_sealed_retain_error() {
+        let sahm = SealableAHashMap::new(AHashMap::from([(0, 0)]));
+        let mut sealed = sahm.seal().unwrap();
+        let err = sealed.retain(|_, _| true).unwrap_err();
+        assert_eq!(err, Errno::EPERM);
+    }
 }
