@@ -69,14 +69,17 @@ use std::{
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
+    fcntl::{open, OFlag},
     libc::setdomainname,
+    pty::{grantpt, posix_openpt, ptsname_r, unlockpt},
     sched::{unshare, CloneFlags},
     sys::{
         socket::{bind, socket, AddressFamily, SockFlag, SockType, SockaddrIn, SockaddrIn6},
+        stat::Mode,
         wait::{Id, WaitPidFlag},
     },
     time::{clock_gettime, ClockId},
-    unistd::{fork, getgid, getpid, getuid, sethostname, ForkResult},
+    unistd::{fork, getgid, getpid, getuid, isatty, sethostname, ForkResult},
     NixPath,
 };
 use sendfd::SendWithFd;
@@ -86,7 +89,7 @@ use syd::{
     config::*,
     err::{err2no, SydResult},
     error,
-    fs::{pidfd_open, set_cloexec},
+    fs::{pidfd_open, retry_on_eintr, set_cloexec},
     hash::SydRandomState,
     hook::Supervisor,
     info,
@@ -427,6 +430,102 @@ fn main() -> SydResult<ExitCode> {
         sandbox.set_unshare_pid(true);
     }
 
+    let pty_debug = env::var_os("SYD_PTY_DEBUG").is_some();
+    let pty_child = if sandbox.has_pty() && isatty(std::io::stdin()).unwrap_or(false) {
+        // Step 1: Create a PIDFd of this process and clear O_CLOEXEC.
+        let pidfd = pidfd_open(getpid(), libc::PIDFD_NONBLOCK)?;
+        set_cloexec(&pidfd, false)?;
+
+        // Step 2: Open a pseudoterminal device without O_CLOEXEC.
+        let pty_m = posix_openpt(OFlag::O_RDWR | OFlag::O_NOCTTY | OFlag::O_NOFOLLOW)?;
+        grantpt(&pty_m)?;
+        unlockpt(&pty_m)?;
+        let pty_s = ptsname_r(&pty_m).map(XPathBuf::from)?;
+        #[allow(clippy::disallowed_methods)]
+        let pty_s = open(
+            &pty_s,
+            OFlag::O_RDWR | OFlag::O_NOCTTY | OFlag::O_NOFOLLOW | OFlag::O_CLOEXEC,
+            Mode::empty(),
+        )?;
+
+        // Step 3: Prepare environment of the syd-pty process.
+        // Filter the environment variables to only include the list below:
+        // 1. PATH
+        // 2. LD_LIBRARY_PATH
+        // 3. SYD_PTY_DEBUG
+        // 4. SYD_PTY_RULES
+        let safe_env: HashMap<_, _, SydRandomState> = env::vars_os()
+            .filter(|(key, _)| {
+                matches!(
+                    key.as_bytes(),
+                    b"PATH" | b"LD_LIBRARY_PATH" | b"SYD_PTY_DEBUG" | b"SYD_PTY_RULES"
+                )
+            })
+            .collect();
+
+        // Step 5: Spawn syd-pty process with a double-fork.
+        // Pass PTY main end to it.
+        let mut cmd = std::process::Command::new(
+            env::var_os("CARGO_BIN_EXE_syd-pty").unwrap_or(OsString::from("syd-pty")),
+        );
+        if pty_debug {
+            cmd.arg("-d");
+        }
+        cmd.arg(format!("-p{}", pidfd.as_raw_fd()));
+        cmd.arg(format!("-i{}", pty_m.as_raw_fd()));
+        if let Some(ws) = sandbox.pty_ws_x() {
+            cmd.arg(format!("-x{ws}"));
+        }
+        if let Some(ws) = sandbox.pty_ws_y() {
+            cmd.arg(format!("-y{ws}"));
+        }
+        cmd.env_clear();
+        cmd.envs(safe_env);
+        cmd.stdin(Stdio::inherit());
+        cmd.stdout(Stdio::inherit());
+        if !pty_debug {
+            cmd.stderr(Stdio::null());
+        }
+        // SAFETY: In libc we trust.
+        match unsafe { fork() }? {
+            ForkResult::Parent { child } => {
+                retry_on_eintr(|| {
+                    waitid(
+                        Id::Pid(child),
+                        WaitPidFlag::WEXITED | WaitPidFlag::__WNOTHREAD,
+                    )
+                })?;
+            }
+            ForkResult::Child => {
+                // SAFETY: In libc we trust.
+                match unsafe { fork() }? {
+                    ForkResult::Parent { .. } => {
+                        // SAFETY: In libc we trust.
+                        unsafe { libc::_exit(0) };
+                    }
+                    ForkResult::Child => return Err(cmd.exec().into()),
+                };
+            }
+        }
+        drop(pidfd);
+        drop(pty_m);
+        if pty_debug {
+            warn!("ctx": "spawn_pty",
+                "msg": "syd-pty is now forwarding terminal I/O");
+        } else {
+            info!("ctx": "spawn_pty",
+                "msg": "syd-pty is now forwarding terminal I/O");
+        }
+
+        // Step 6: Pass the other end of the PTY pair
+        // to the sandbox process.
+        env::set_var(ENV_PTY_FD, pty_s.as_raw_fd().to_string());
+        Some(pty_s)
+    } else {
+        env::remove_var(ENV_PTY_FD);
+        None
+    };
+
     let proxy_debug = env::var_os("SYD_TOR_DEBUG").is_some();
     let proxy = if sandbox.has_proxy() {
         // sandbox/proxy:on implies unshare/net:1
@@ -514,7 +613,14 @@ fn main() -> SydResult<ExitCode> {
     let namespaces = sandbox.namespaces();
     if namespaces == 0 {
         // No namespace arguments passed, run normally.
-        return match Supervisor::run(sandbox, &argv0, cmd_argv, Some(&cmd_envp), cmd_arg0) {
+        return match Supervisor::run(
+            sandbox,
+            pty_child,
+            &argv0,
+            cmd_argv,
+            Some(&cmd_envp),
+            cmd_arg0,
+        ) {
             Ok(code) => Ok(ExitCode::from(code)),
             Err(error) => {
                 let errno = Errno::last();
@@ -730,8 +836,14 @@ fn main() -> SydResult<ExitCode> {
     // SAFETY: fork is our safest option here.
     let child = match unsafe { fork() }? {
         ForkResult::Child => {
-            let retval = match Supervisor::run(sandbox, &argv0, cmd_argv, Some(&cmd_envp), cmd_arg0)
-            {
+            let retval = match Supervisor::run(
+                sandbox,
+                pty_child,
+                &argv0,
+                cmd_argv,
+                Some(&cmd_envp),
+                cmd_arg0,
+            ) {
                 Ok(retval) => libc::c_int::from(retval),
                 Err(error) => {
                     let errno = Errno::last();
@@ -747,6 +859,7 @@ fn main() -> SydResult<ExitCode> {
             drop(cmd_arg0);
             drop(cmd_argv);
             drop(cmd_envp);
+            drop(pty_child);
             drop(sandbox);
             // SAFETY: nix does not have a close_range wrapper yet.
             let _ = unsafe {
