@@ -32,8 +32,6 @@
 //
 // If the "log" feature is not enabled, we provide stubs that return ENOSYS.
 
-use std::ffi::CString;
-
 use btoi::btoi;
 use memchr::arch::all::is_equal;
 
@@ -221,23 +219,6 @@ pub const SYSLOG_ACTION_SIZE_UNREAD: libc::c_int = 9;
 /// buffer.  The bufp and len arguments are ignored.
 pub const SYSLOG_ACTION_SIZE_BUFFER: libc::c_int = 10;
 
-/// Logs a message to the system's syslog.
-pub fn host_syslog(message: &str) {
-    let c_msg = CString::new(message).unwrap_or_else(|_|
-        // SAFETY: We ensure the string has no null bytes and the vector
-        // we pass into the function is an actual byte vector.
-        unsafe { CString::from_vec_unchecked(b"?".to_vec()) });
-
-    // SAFETY: Use the syslog interface provided by libc.
-    unsafe {
-        nix::libc::syslog(
-            nix::libc::LOG_WARNING,
-            c"%s".as_ptr().cast(),
-            c_msg.as_ptr(),
-        );
-    }
-}
-
 // LOG-ENABLED IMPLEMENTATION
 #[cfg(feature = "log")]
 mod syslog_enabled {
@@ -253,6 +234,7 @@ mod syslog_enabled {
 
     use nix::{
         errno::Errno,
+        syslog::{openlog, syslog, Facility, LogFlags, Priority, Severity},
         time::{clock_gettime, ClockId},
     };
     use ringbuf::{
@@ -264,6 +246,39 @@ mod syslog_enabled {
 
     use crate::{config::SYSLOG_STACK_SIZE, log::LockedWriter, syslog::*};
 
+    /// Opens a socket connection to the system's syslog.
+    ///
+    /// Note, if this connection ever gets broken the
+    /// subsequent `host_syslog` call is going to cause
+    /// instant termination of the Syd process and all
+    /// processes running under it. This may be an issue
+    /// long-term, therefore:
+    ///
+    /// TODO: Implement our own system.syslog service
+    ///       a la Capsicum's libcasper.
+    fn open_syslog() -> Result<(), Errno> {
+        // LOG_PID: Log process ID.
+        // LOG_NDELAY: Open connection to syslogd(8) immediately,
+        //             because the default behaviour of openlog(3)
+        //             is to delay opening the connection until
+        //             the first `host_syslog` call in which case
+        //             our socket(2) seccomp filters will gladly
+        //             terminate us.
+        openlog(
+            Some(c"syd"),
+            LogFlags::LOG_PID | LogFlags::LOG_NDELAY,
+            Facility::LOG_USER,
+        )
+    }
+
+    /// Logs a message to the system's syslog.
+    fn host_syslog(message: &str) -> Result<(), Errno> {
+        syslog(
+            Priority::new(Severity::LOG_WARNING, Facility::LOG_USER),
+            message,
+        )
+    }
+
     // Store ring data in an enum to handle "heap" vs "static" capacity.
     // Store Arc<SharedRb<...>> plus Caching wrappers to fix all trait bounds.
     enum RbMode {
@@ -374,8 +389,20 @@ mod syslog_enabled {
             // Always write to host syslog if log level is Warn or higher.
             // Do not use formatting for syslog messages.
             if self.host_syslog() && level.as_u8() <= LogLevel::Warn.as_u8() {
-                // Call host syslog with no formatting.
-                host_syslog(msg);
+                // Call host syslog with no formatting:
+                //
+                // SAFETY:
+                // 1. We use openlog(3) at startup with LOG_NDELAY,
+                //    and an attempt to reconnect to the syslogd(3) via
+                //    syslog(3) will cause instant Syd termination together
+                //    with all the processes running under it.
+                // 2. Terminating the process here with a panic is arguably
+                //    the best we can do given we do not want to continue
+                //    without the host visibility and next socket(2) is doomed
+                //    to kill us anyway.
+                //    TODO: system.syslog service a la Capsicum's libcasper.
+                #[allow(clippy::disallowed_methods)]
+                host_syslog(msg).expect("syslogd");
             }
 
             // Always attempt to write message to fd, append a new line to the message.
@@ -803,6 +830,13 @@ mod syslog_enabled {
             return Err(Errno::EINVAL);
         }
 
+        if host_syslog {
+            // SAFETY: Open syslogd(8) connection immediately with LOG_NDELAY,
+            // to ensure we can continue sending syslog(3) messages
+            // under seccomp(2) confinement with syscall cookies et al.
+            open_syslog()?;
+        }
+
         SYSLOG_INSTANCE
             .set(Syslog::new(capacity, fd, level, host_syslog, use_stack))
             .or(Err(Errno::EAGAIN))
