@@ -313,6 +313,19 @@ macro_rules! skip_if_root {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_stdin_is_a_tty {
+    () => {{
+        if !nix::unistd::isatty(std::io::stdin()).unwrap_or(false) {
+            eprintln!("Test requires STDIN to be a TTY, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!("STDIN is a TTY, proceeding with test...");
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_kernel_crypto_is_supported {
     () => {{
@@ -637,7 +650,11 @@ pub struct Syd(Command);
 
 impl Syd {
     pub fn new(cmd: &str) -> Self {
-        Syd(Command::new(cmd))
+        let mut cmd = Command::new(cmd);
+
+        cmd.stdin(Stdio::null());
+
+        Syd(cmd)
     }
 
     pub fn c<S: ToString>(&mut self, arg: S) -> &mut Self {
@@ -715,12 +732,10 @@ impl Syd {
         self
     }
 
-    /*
     pub fn stdin(&mut self, cfg: Stdio) -> &mut Self {
         self.0.stdin(cfg);
         self
     }
-    */
 
     pub fn stdout(&mut self, cfg: Stdio) -> &mut Self {
         self.0.stdout(cfg);
