@@ -11,23 +11,26 @@ use std::{
 
 use btoi::btoi;
 use libloading::os::unix::Symbol;
+use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext};
 use nix::{
     errno::Errno,
     libc,
     sys::signal::{kill, sigprocmask, SigSet, SigmaskHow, Signal},
-    unistd::{close, getpid, read, write},
+    unistd::{close, getpid, read, setsid, tcsetpgrp, write},
 };
 
 use crate::{
     caps,
     config::*,
-    fs::retry_on_eintr,
+    err::SydResult,
+    fs::{retry_on_eintr, SECCOMP_IOCTL_NOTIF_LIST},
     landlock::{CompatLevel, Compatible, Ruleset, RulesetAttr, Scope},
-    safe_drop_cap,
+    safe_drop_cap, seccomp_add_architectures,
     unshare::{
         error::ErrorCode as Err,
         run::{ChildInfo, Exe},
     },
+    ScmpSyscall,
 };
 
 unsafe fn fail_errno(code: Err, errno: i32) -> ! {
@@ -39,8 +42,13 @@ unsafe fn fail_errno(code: Err, errno: i32) -> ! {
         Err::ProcessStop => c"syd: error stopping process".as_ptr(),
         Err::ResetSignal => c"syd: error reseting signals".as_ptr(),
         Err::Seccomp => c"syd: seccomp error".as_ptr(),
+        Err::SeccompKillFd => c"syd: seccomp kill notify-ioctl error".as_ptr(),
         Err::SeccompSendFd => c"syd: seccomp send notify-fd error".as_ptr(),
         Err::SeccompWaitFd => c"syd: seccomp wait for notify-fd error".as_ptr(),
+        Err::SetSid => c"syd: setsid error".as_ptr(),
+        Err::SetPty => c"syd: error setting pty as controlling terminal".as_ptr(),
+        Err::DupPty => c"syd: error duplicating pty onto stdio fds".as_ptr(),
+        Err::SetPgrp => c"syd: error setting foreground process group".as_ptr(),
         #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
         Err::SetTSC => c"syd: set-tsc error".as_ptr(),
     };
@@ -64,7 +72,42 @@ macro_rules! fail_errno_safe {
 
 #[allow(clippy::cognitive_complexity)]
 pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
-    // SAFETY: Apply a Landlock scope sandbox to restrict
+    // SAFETY: arg is a valid ChildInfo structure.
+    let mut child: Box<ChildInfo> = unsafe { Box::from_raw(arg as *mut ChildInfo) };
+
+    // Restriction 0: Change controlling terminal to PTY as necessary.
+    if let Some(pty_fd) = child.pty_fd.take() {
+        // Become session leader so we can take a controlling TTY.
+        if let Err(errno) = setsid() {
+            fail_errno_safe!(child, Err::SetSid, errno as i32);
+        }
+
+        // Make the PTY fd our controlling terminal.
+        if let Err(errno) =
+            Errno::result(unsafe { libc::ioctl(pty_fd.as_raw_fd(), libc::TIOCSCTTY, 0) })
+        {
+            fail_errno_safe!(child, Err::SetPty, errno as i32);
+        }
+
+        // Make us the foreground process group.
+        if let Err(errno) = tcsetpgrp(&pty_fd, getpid()) {
+            fail_errno_safe!(child, Err::SetPgrp, errno as i32);
+        }
+
+        // Duplicate PTY fd onto stdio(3) fds.
+        for std_fd in [libc::STDIN_FILENO, libc::STDOUT_FILENO, libc::STDERR_FILENO] {
+            if let Err(errno) = Errno::result(unsafe { libc::dup2(pty_fd.as_raw_fd(), std_fd) }) {
+                fail_errno_safe!(child, Err::DupPty, errno as i32);
+            }
+        }
+
+        // Close the original PTY fd.
+        drop(pty_fd);
+    }
+
+    // Restriction 1:
+    //
+    // Apply a Landlock scope sandbox to restrict
     // 1. Ptrace attach outside Landlock.
     // 2. Signal send outside Landlock.
     // 3. UNIX abstract socket connect outside Landlock.
@@ -80,6 +123,52 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
         let _ = ruleset.restrict_self();
     }
 
+    // Restriction 2: Disallow ioctl seccomp notify family requests.
+    // We deny these regardless of trace/allow_unsafe_cbpf:1 because
+    // even when these requests are allowed it is not securely possible
+    // to stack to seccomp-notify sandboxes, one of them being Syd.
+    //
+    // SAFETY: Be swift and KillProcess rather than Deny for these cases.
+    if let Ok(syscall) = ScmpSyscall::from_name("ioctl") {
+        let result = (|| -> SydResult<()> {
+            let mut ctx = ScmpFilterContext::new(ScmpAction::Allow)?;
+
+            // Enforce the NO_NEW_PRIVS functionality before
+            // loading the seccomp filter into the kernel.
+            ctx.set_ctl_nnp(true)?;
+            // Enable Speculative Store Bypass mitigations.
+            ctx.set_ctl_ssb(false)?;
+            // Do not synchronize filter to all threads.
+            ctx.set_ctl_tsync(false)?;
+            // We deny with ENOSYS for bad/unsupported system call,
+            // and kill process for bad arch.
+            ctx.set_act_badarch(ScmpAction::KillProcess)?;
+            // Use a binary tree sorted by syscall number if possible.
+            let _ = ctx.set_ctl_optimize(2);
+            // We don't want ECANCELED, we want actual errnos.
+            let _ = ctx.set_api_sysrawrc(true);
+
+            seccomp_add_architectures(&mut ctx)?;
+
+            for request in SECCOMP_IOCTL_NOTIF_LIST {
+                ctx.add_rule_conditional(
+                    ScmpAction::KillProcess,
+                    syscall,
+                    &[scmp_cmp!($arg1 == *request)],
+                )?;
+            }
+
+            ctx.load()?;
+
+            Ok(())
+        })()
+        .map_err(|err| err.errno().unwrap_or(Errno::EOWNERDEAD));
+
+        if let Err(errno) = result {
+            fail_errno_safe!(child, Err::SeccompKillFd, errno as i32);
+        }
+    }
+
     // SAFETY: Do not leak the static file descriptors to the sandbox process.
     proc_close();
 
@@ -108,9 +197,6 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
         }
     }
 
-    // SAFETY: arg is a valid ChildInfo structure.
-    let child: Box<ChildInfo> = unsafe { Box::from_raw(arg as *mut ChildInfo) };
-
     // We'll write seccomp notify fd to the second pipe,
     // and read the acknowledgement notification from
     // the first pipe.
