@@ -11,10 +11,15 @@ pub enum ErrorCode {
     ProcessStop = 5,
     ResetSignal = 6,
     Seccomp = 7,
-    SeccompSendFd = 8,
-    SeccompWaitFd = 9,
+    SeccompKillFd = 8,
+    SeccompSendFd = 9,
+    SeccompWaitFd = 10,
+    SetSid = 11,
+    SetPty = 12,
+    DupPty = 13,
+    SetPgrp = 14,
     #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
-    SetTSC = 10,
+    SetTSC = 15,
 }
 
 /// Error runnning process
@@ -48,10 +53,20 @@ pub enum Error {
     ResetSignal(i32),
     /// Seccomp error (loading filter, getting notify fd)
     Seccomp(i32),
+    /// Error denying seccomp notify ioctl(2) requests
+    SeccompKillFd(i32),
     /// Error sending notification fd through the seccomp sender channel
     SeccompSendFd(i32),
     /// Error waiting for parent to receive the seccomp fd
     SeccompWaitFd(i32),
+    /// Error calling setsid(2)
+    SetSid(i32),
+    /// Error calling TIOCSCTTY ioctl(2)
+    SetPty(i32),
+    /// Error calling dup(2) on PTY fd
+    DupPty(i32),
+    /// Error calling tcsetpgrp(3)
+    SetPgrp(i32),
     /// Error calling prctl PR_SET_TSC
     #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
     SetTSC(i32),
@@ -74,8 +89,13 @@ impl Error {
             ProcessStop(x) => Some(x),
             ResetSignal(x) => Some(x),
             Seccomp(x) => Some(x),
+            SeccompKillFd(x) => Some(x),
             SeccompSendFd(x) => Some(x),
             SeccompWaitFd(x) => Some(x),
+            SetSid(x) => Some(x),
+            SetPty(x) => Some(x),
+            DupPty(x) => Some(x),
+            SetPgrp(x) => Some(x),
             #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
             SetTSC(x) => Some(x),
         }
@@ -96,8 +116,13 @@ impl Error {
             ProcessStop(_) => "error stopping process",
             ResetSignal(_) => "error reseting signals",
             Seccomp(_) => "error in seccomp filter load",
+            SeccompKillFd(_) => "error killing seccomp notify ioctls",
             SeccompSendFd(_) => "error sending seccomp file descriptor",
             SeccompWaitFd(_) => "error waiting for parent to receive the seccomp file descriptor",
+            SetSid(_) => "error calling setsid",
+            SetPty(_) => "error setting pty as controlling terminal",
+            DupPty(_) => "error duplicating pty onto stdio fds",
+            SetPgrp(_) => "error setting foreground process group",
             #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
             SetTSC(_) => "error setting timestamp counter prctl",
         }
