@@ -146,6 +146,7 @@ fn list() {
     println!("core");
     println!("debug");
     println!("enforce");
+    println!("firefox");
     println!("immutable");
     println!("landlock");
     println!("lib");
@@ -338,6 +339,15 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "ff" | "firefox" => {
+            println!("# Syd profile: Firefox");
+            println!("# Number of rules: {}", syd::config::PROFILE_FIREFOX.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_FIREFOX {
+                println!("{command}");
+            }
+        }
         "off" => {
             println!("# Syd profile: Off");
             println!("# Number of rules: {}", syd::config::PROFILE_OFF.len());
