@@ -180,7 +180,7 @@ impl Executor for SydExecutor {
 
         // Run command under the Syd sandbox and exit with the return code.
         #[allow(clippy::disallowed_methods)]
-        Supervisor::run(sandbox, comm, argv, None, None)
+        Supervisor::run(sandbox, None, comm, argv, None, None)
             .map(i32::from)
             .map(exit)
             .map_err(|err| ExecutorError::Execution(err.into()))?
