@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-tor.rs: Syd's SOCKS Proxy Forwarder
+// src/utils/syd-tor.rs: Syd's SOCKS Proxy Forwarder
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 // Based in part upon socksns crate which is:
@@ -29,9 +29,7 @@ use nix::{
     sched::{unshare, CloneFlags},
     sys::{
         epoll::{Epoll, EpollCreateFlags, EpollEvent, EpollFlags},
-        prctl::set_pdeathsig,
         resource::{getrlimit, setrlimit, Resource},
-        signal::Signal,
         socket::{
             accept4, connect, listen, socket, AddressFamily, Backlog, SockFlag, SockType,
             SockaddrLike, SockaddrStorage, UnixAddr,
@@ -97,7 +95,7 @@ fn main() -> SydResult<ExitCode> {
     }
     // SAFETY: nix does not have a close_range wrapper yet.
     // Nothing can be done on close_range errors.
-    let _ = unsafe { libc::syscall(libc::SYS_close_range, max_fd + 1, libc::c_int::MAX, 0) };
+    let _ = unsafe { libc::syscall(libc::SYS_close_range, max_fd + 1, RawFd::MAX, 0) };
 
     // SAFETY: We leak the UNIX socket fd on purpose,
     // and leave it open for process lifetime.
@@ -168,9 +166,12 @@ fn main() -> SydResult<ExitCode> {
     confine(&addr, debug, print)?;
 
     // Run the proxy server.
-    run_proxy_server(client_manager, &epoll, &fpid, &proxy, &addr, pmax, debug).expect("TOR");
+    let result = run_proxy_server(client_manager, &epoll, &fpid, &proxy, &addr, pmax, debug);
 
-    Ok(ExitCode::SUCCESS)
+    Ok(match result {
+        Ok(_) => ExitCode::SUCCESS,
+        Err(err) => ExitCode::from(err.errno().unwrap_or(Errno::ENOSYS) as i32 as u8),
+    })
 }
 
 /// Run the proxy server
@@ -712,12 +713,8 @@ fn confine(addr: &SockaddrStorage, dry_run: bool, print_rules: bool) -> SydResul
     chdir(safe_cwd)?;
 
     if !dry_run {
-        // Set parent-death signal to SIGKILL.
-        // We do not want to outlive the caller Syd process.
-        // Since we do have a pidfd to the Syd process,
-        // and this is the second layer, we ignore errors
-        // and move on.
-        let _ = set_pdeathsig(Some(Signal::SIGKILL));
+        // SAFETY: Default panic hook wont play well with seccomp.
+        std::panic::set_hook(Box::new(|_| {}));
 
         // Set up namespace isolation.
         // The namespaces to create depends on the type of the external address.
