@@ -34,6 +34,7 @@ use crate::{
     },
     info,
     path::XPathBuf,
+    sandbox::Flags,
     ExportMode,
 };
 
@@ -74,25 +75,17 @@ pub(crate) type AesMap = Arc<RwLock<(HashMap<XPathBuf, AesVal, SydRandomState>,
 pub(crate) struct AesWorker {
     fdalg: (RawFd, RawFd),
     files: AesMap,
+    flags: Flags,
     is_memfd: bool,
-    safe_setid: bool,
-    scmp_ssb: bool,
 }
 
 impl AesWorker {
-    pub(crate) fn new(
-        fdalg: (RawFd, RawFd),
-        files: AesMap,
-        is_memfd: bool,
-        safe_setid: bool,
-        scmp_ssb: bool,
-    ) -> Self {
+    pub(crate) fn new(fdalg: (RawFd, RawFd), files: AesMap, flags: Flags, is_memfd: bool) -> Self {
         Self {
             fdalg,
             files,
+            flags,
             is_memfd,
-            safe_setid,
-            scmp_ssb,
         }
     }
 
@@ -112,14 +105,19 @@ impl AesWorker {
 
                 // SAFETY: Panic if we cannot prepare the seccomp filter.
                 #[allow(clippy::disallowed_methods)]
-                let ctx = Self::prepare_confine(self.safe_setid, self.scmp_ssb)
-                    .expect("prepare AES thread confinement");
+                let ctx =
+                    Self::prepare_confine(self.flags).expect("prepare AES thread confinement");
+
                 #[allow(clippy::disallowed_methods)]
                 if !dry_run {
                     // SAFETY: Logging will kill us after seccomp.
+                    let safe_setid = self
+                        .flags
+                        .intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
                     info!("ctx": "confine", "op": "confine_aes_thread",
                         "msg": format!("AES thread confined with{} SROP mitigation",
-                            if self.safe_setid { "out" } else { "" }));
+                            if safe_setid { "out" } else { "" }));
+
                     // SAFETY: Panic if we cannot confine the thread.
                     ctx.load().expect("confine AES thread");
                 } else {
@@ -638,7 +636,7 @@ impl AesWorker {
 
     /// Confine AES thread.
     #[allow(clippy::cognitive_complexity)]
-    pub(crate) fn prepare_confine(safe_setid: bool, ssb: bool) -> SydResult<ScmpFilterContext> {
+    pub(crate) fn prepare_confine(flags: Flags) -> SydResult<ScmpFilterContext> {
         // Create seccomp filter with default action.
         let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
 
@@ -648,7 +646,7 @@ impl AesWorker {
 
         // Disable Speculative Store Bypass mitigations
         // with trace/allow_unsafe_spec_exec:1
-        ctx.set_ctl_ssb(ssb)?;
+        ctx.set_ctl_ssb(flags.contains(Flags::FL_ALLOW_UNSAFE_SPEC_EXEC))?;
 
         // DO NOT synchronize filter to all threads.
         // Main thread will confine itself.
@@ -762,7 +760,7 @@ impl AesWorker {
             }
         }
 
-        if safe_setid {
+        if flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID) {
             // SAFETY: Main thread confines these further.
             // As these system calls as per-process,
             // the main thread's seccomp rules will apply
