@@ -22,32 +22,47 @@ use std::{
     thread::JoinHandle,
 };
 
-use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
-use libseccomp_sys::{seccomp_notif_resp, seccomp_notify_receive, seccomp_notify_respond};
+use libseccomp::{scmp_cmp, RawSyscall, ScmpAction, ScmpFilterContext, ScmpSyscall};
+use libseccomp_sys::seccomp_notify_receive;
 use nix::{
     errno::Errno,
     poll::PollTimeout,
     sched::{sched_setaffinity, unshare, CloneFlags, CpuSet},
-    sys::epoll::{Epoll, EpollEvent},
+    sys::epoll::{Epoll, EpollEvent, EpollFlags},
     unistd::{close, Pid},
 };
 
+#[cfg(target_arch = "x86")]
+use crate::cookie::FTRUNCATE64_COOKIE_ARG3;
+#[cfg(target_arch = "x86")]
+use crate::cookie::TRUNCATE64_COOKIE_ARG3;
 use crate::{
     compat::epoll_ctl_safe,
     config::*,
     cookie::{
+        FTRUNCATE64_COOKIE_ARG4, FTRUNCATE64_COOKIE_ARG5, FTRUNCATE_COOKIE_ARG2,
+        FTRUNCATE_COOKIE_ARG3, FTRUNCATE_COOKIE_ARG4, FTRUNCATE_COOKIE_ARG5,
         MEMFD_CREATE_COOKIE_ARG2, MEMFD_CREATE_COOKIE_ARG3, MEMFD_CREATE_COOKIE_ARG4,
         MEMFD_CREATE_COOKIE_ARG5, OPENAT2_COOKIE_ARG4, OPENAT2_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5,
-        UNLINKAT_COOKIE_ARG3, UNLINKAT_COOKIE_ARG4, UNLINKAT_COOKIE_ARG5,
+        SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4,
+        SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3,
+        SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5,
+        SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4, SOCKET_COOKIE_ARG5, SYS_SOCKET,
+        TRUNCATE64_COOKIE_ARG4, TRUNCATE64_COOKIE_ARG5, TRUNCATE_COOKIE_ARG2, TRUNCATE_COOKIE_ARG3,
+        TRUNCATE_COOKIE_ARG4, TRUNCATE_COOKIE_ARG5, UNLINKAT_COOKIE_ARG3, UNLINKAT_COOKIE_ARG4,
+        UNLINKAT_COOKIE_ARG5,
     },
     err::{err2no, SydResult},
     error, extend_ioctl,
-    fs::AT_EXECVE_CHECK,
-    hook::{HandlerMap, UNotifyEventRequest, SECCOMP_IOCTL_LIST},
+    fs::{
+        seccomp_notify_respond, AT_EXECVE_CHECK, SECCOMP_IOCTL_NOTIF_ADDFD,
+        SECCOMP_IOCTL_NOTIF_LIST, SECCOMP_IOCTL_NOTIF_SEND,
+    },
+    hook::{HandlerMap, UNotifyEventRequest},
     info,
     path::dotdot_with_nul,
     proc::proc_mmap,
-    sandbox::{Sandbox, SandboxGuard},
+    sandbox::{Flags, Sandbox, SandboxGuard},
     scmp_arch_raw,
     workers::{aes::AesMap, BindMap, WorkerCache, WorkerData},
     ExportMode, ScmpNotifReq, SydArch, Sydcall,
@@ -59,9 +74,7 @@ const EOWNERDEAD: i32 = -libc::EOWNERDEAD;
 pub(crate) struct Worker {
     fd: RawFd,
     child: Pid,
-    safe_setid: bool,
-    scmp_ssb: bool,
-    wait_all: bool,
+    flags: Flags,
     epoll: Arc<Epoll>,
     cache: Arc<WorkerCache<'static>>,
     sandbox: Arc<RwLock<Sandbox>>,
@@ -89,17 +102,13 @@ impl Worker {
     ) -> Self {
         let my_sandbox = SandboxGuard::Read(sandbox.read().unwrap_or_else(|err| err.into_inner()));
         let child = my_sandbox.get_child_pid();
-        let wait_all = my_sandbox.exit_wait_all();
-        let safe_setid = my_sandbox.allow_safe_setuid() || my_sandbox.allow_safe_setgid();
-        let scmp_ssb = my_sandbox.allow_unsafe_spec_exec();
+        let flags = *my_sandbox.flags;
         drop(my_sandbox); // release the read lock.
 
         Worker {
             fd,
             child,
-            safe_setid,
-            scmp_ssb,
-            wait_all,
+            flags,
             epoll,
             cache,
             sandbox,
@@ -180,15 +189,15 @@ impl Worker {
                         let ctx = Self::prepare_confine(
                             self.fd,
                             self.epoll.0.as_raw_fd(),
-                            self.safe_setid,
-                            self.scmp_ssb,
+                            self.flags,
                         ).expect("prepare emulator thread confinement");
 
                         // Honour dry-run when exporting.
                         if !dry_run {
+                            let safe_setid = self.flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
                             info!("ctx": "confine", "op": "confine_emulator_thread",
                                 "msg": format!("emulator thread confined with{} SROP mitigation",
-                                    if self.safe_setid { "out" } else { "" }));
+                                    if safe_setid { "out" } else { "" }));
 
                             // SAFETY: Panic if we cannot confine the thread.
                             #[allow(clippy::disallowed_methods)]
@@ -291,9 +300,8 @@ impl Worker {
         let response = handler(request);
 
         if response.id == 0 && response.val == 0 && response.error == 0 && response.flags == 0 {
-            // Dummy seccomp response.
-            // A previous addfd request has
-            // already replied to the request.
+            // All-zero dummy empty seccomp response.
+            // A previous ADDFD request has already replied to the request.
             // Nothing left to do here.
             return;
         } else if response.error == EOWNERDEAD {
@@ -306,20 +314,17 @@ impl Worker {
                 "src": proc_mmap(req.pid(), req.data.instr_pointer).ok());
         }
 
-        let mut response = libc::seccomp_notif_resp {
+        let response = libc::seccomp_notif_resp {
             id: response.id,
             val: response.val,
             error: response.error,
             flags: response.flags,
         };
-        // libc's data type is not opaque unlike libseccomp's.
-        let ptr: *mut seccomp_notif_resp =
-            std::ptr::addr_of_mut!(response) as *mut seccomp_notif_resp;
 
-        // SAFETY:
-        // 1. libseccomp's version allocates needlessly
-        // 2. Nothing we can do on errors, EINTR means child dead.
-        unsafe { seccomp_notify_respond(self.fd, ptr) };
+        // EAGAIN|EINTR is retried.
+        // ENOENT means child died mid-way.
+        // Nothing else we can do on errors here.
+        let _ = seccomp_notify_respond(self.fd, std::ptr::addr_of!(response));
 
         // See above.
         if response.error == EOWNERDEAD {
@@ -355,7 +360,29 @@ impl Worker {
             match self.epoll.wait(&mut events, timeout) {
                 Ok(0) if self.keep_alive.is_some() => return Err(Errno::ETIMEDOUT),
                 Ok(0) | Err(Errno::EINTR) => {} // try again.
-                Ok(1) if events[0].data() == 0 => return Ok(()), // scmp-req.
+                Ok(1) if events[0].data() == 0 => {
+                    // Seccomp request.
+                    let flags = events[0].events();
+
+                    return if flags.contains(EpollFlags::EPOLLIN) {
+                        // Received scmp request.
+                        Ok(())
+                    } else if flags.intersects(
+                        EpollFlags::EPOLLERR | EpollFlags::EPOLLHUP | EpollFlags::EPOLLRDHUP,
+                    ) {
+                        // Exit on any hang-up or error.
+                        //
+                        // Inform the monitor thread to exit.
+                        self.should_exit.store(true, Ordering::Relaxed);
+
+                        Err(Errno::EPIPE)
+                    } else {
+                        unreachable!(
+                            "BUG: seccomp request event with invalid flags {:#x}!",
+                            flags.bits()
+                        );
+                    };
+                }
                 Ok(1) => {
                     // Record retired PID and try again.
                     // These fds are added with EPOLLONESHOT.
@@ -367,7 +394,7 @@ impl Worker {
                         map.del_pidfd(pid);
                     }
 
-                    if !self.wait_all && pid == self.child {
+                    if !self.flags.contains(Flags::FL_EXIT_WAIT_ALL) && pid == self.child {
                         // Note, threads blocked on epoll_wait will not
                         // wake up even if we close the epoll fd or
                         // delete the seccomp fd from epoll wait-list here.
@@ -396,8 +423,7 @@ impl Worker {
     pub(crate) fn prepare_confine(
         seccomp_fd: RawFd,
         epoll_fd: RawFd,
-        safe_setid: bool,
-        ssb: bool,
+        flags: Flags,
     ) -> SydResult<ScmpFilterContext> {
         // Create seccomp filter with default action.
         let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
@@ -408,7 +434,7 @@ impl Worker {
 
         // Disable Speculative Store Bypass mitigations
         // with trace/allow_unsafe_spec_exec:1
-        ctx.set_ctl_ssb(ssb)?;
+        ctx.set_ctl_ssb(flags.contains(Flags::FL_ALLOW_UNSAFE_SPEC_EXEC))?;
 
         // DO NOT synchronize filter to all threads.
         // Main thread will confine itself.
@@ -435,41 +461,65 @@ impl Worker {
                     ctx.add_rule(ScmpAction::Errno(Errno::ENOSYS as i32), syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
         }
 
         // Allow a restricted set of ioctl(2) operations to our seccomp fd only.
+        //
+        // Syscall argument cookies for SECCOMP_IOCTL_NOTIF_SEND my be disabled
+        // at startup with trace/allow_unsafe_nocookie:1.
         let sysname = "ioctl";
+        let restrict_cookie = !flags.contains(Flags::FL_ALLOW_UNSAFE_NOCOOKIE);
         #[allow(clippy::cast_sign_loss)]
         match ScmpSyscall::from_name(sysname) {
             Ok(syscall) => {
-                for ioctl_request in SECCOMP_IOCTL_LIST {
-                    ctx.add_rule_conditional(
-                        ScmpAction::Allow,
-                        syscall,
-                        &[
-                            scmp_cmp!($arg0 == seccomp_fd as u64),
-                            scmp_cmp!($arg1 == *ioctl_request),
-                        ],
-                    )?;
+                for ioctl_request in SECCOMP_IOCTL_NOTIF_LIST {
+                    let mut rules = vec![scmp_cmp!($arg0 == seccomp_fd as u64)];
+
+                    // SAFETY: We protect SECCOMP_IOCTL_NOTIF_ADDFD with system call argument
+                    // cookies, to raise the bar against an attacker who has compromised Syd
+                    // and aims steal file descriptors.
+                    //
+                    // Randomizing the seccomp-fd at startup is another mitigation against this.
+                    #[allow(clippy::useless_conversion)]
+                    if restrict_cookie && *ioctl_request == SECCOMP_IOCTL_NOTIF_ADDFD {
+                        rules.extend(&[
+                            scmp_cmp!($arg3 == (*SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5).into()),
+                        ]);
+                    }
+
+                    // SAFETY: We protect SECCOMP_IOCTL_NOTIF_SEND with system call argument
+                    // cookies, to raise the bar against an attacker who has compromised Syd
+                    // and aims to inject the flag SECCOMP_USER_NOTIF_FLAG_CONTINUE to this
+                    // response in order to pass-through a system call to the host Linux kernel.
+                    //
+                    // Randomizing the seccomp-fd at startup is another mitigation against this.
+                    #[allow(clippy::useless_conversion)]
+                    if restrict_cookie && *ioctl_request == SECCOMP_IOCTL_NOTIF_SEND {
+                        rules.extend(&[
+                            scmp_cmp!($arg3 == (*SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5).into()),
+                        ]);
+                    }
+
+                    rules.push(scmp_cmp!($arg1 == *ioctl_request));
+                    ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
 
                     if let Some(ioctl_request) = extend_ioctl(*ioctl_request) {
-                        ctx.add_rule_conditional(
-                            ScmpAction::Allow,
-                            syscall,
-                            &[
-                                scmp_cmp!($arg0 == seccomp_fd as u64),
-                                scmp_cmp!($arg1 == ioctl_request),
-                            ],
-                        )?;
+                        rules.pop();
+                        rules.push(scmp_cmp!($arg1 == ioctl_request));
+                        ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
                     }
                 }
             }
             Err(_) => {
-                info!("ctx": "confine", "op": "allow_syscall",
+                info!("ctx": "confine", "op": "allow_emu_syscall",
                     "msg": format!("invalid or unsupported syscall {sysname}"));
             }
         }
@@ -486,7 +536,7 @@ impl Worker {
                     )?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -503,7 +553,7 @@ impl Worker {
                 )?;
             }
             Err(_) => {
-                info!("ctx": "confine", "op": "allow_syscall",
+                info!("ctx": "confine", "op": "allow_emu_syscall",
                     "msg": format!("invalid or unsupported syscall {sysname}"));
             }
         }
@@ -527,7 +577,7 @@ impl Worker {
                 )?;
             }
             Err(_) => {
-                info!("ctx": "confine", "op": "allow_syscall",
+                info!("ctx": "confine", "op": "allow_emu_syscall",
                     "msg": format!("invalid or unsupported syscall {sysname}"));
             }
         }
@@ -556,7 +606,7 @@ impl Worker {
                     )?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -567,7 +617,7 @@ impl Worker {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -578,7 +628,7 @@ impl Worker {
             let syscall = match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => syscall,
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                     continue;
                 }
@@ -596,97 +646,240 @@ impl Worker {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
         }
 
-        // memfd_create(2) may be used only with syscall argument cookies.
-        let sysname = "memfd_create";
-        #[allow(clippy::useless_conversion)]
-        match ScmpSyscall::from_name(sysname) {
-            Ok(syscall) => {
-                ctx.add_rule_conditional(
-                    ScmpAction::Allow,
-                    syscall,
-                    &[
-                        scmp_cmp!($arg2 == (*MEMFD_CREATE_COOKIE_ARG2).into()),
-                        scmp_cmp!($arg3 == (*MEMFD_CREATE_COOKIE_ARG3).into()),
-                        scmp_cmp!($arg4 == (*MEMFD_CREATE_COOKIE_ARG4).into()),
-                        scmp_cmp!($arg5 == (*MEMFD_CREATE_COOKIE_ARG5).into()),
-                    ],
-                )?;
+        // Syscall argument cookies may be disabled
+        // at startup with trace/allow_unsafe_nocookie:1.
+        if restrict_cookie {
+            // memfd_create(2) may be used only with syscall argument cookies.
+            let sysname = "memfd_create";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg2 == (*MEMFD_CREATE_COOKIE_ARG2).into()),
+                            scmp_cmp!($arg3 == (*MEMFD_CREATE_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*MEMFD_CREATE_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*MEMFD_CREATE_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
             }
-            Err(_) => {
-                info!("ctx": "confine", "op": "allow_main_syscall",
-                    "msg": format!("invalid or unsupported syscall {sysname}"));
-            }
-        }
 
-        // renameat2(2) may be used only with syscall argument cookies.
-        // We also prevent AT_FDCWD usage as fd argument.
-        let sysname = "renameat2";
-        #[allow(clippy::useless_conversion)]
-        match ScmpSyscall::from_name(sysname) {
-            Ok(syscall) => {
-                ctx.add_rule_conditional(
-                    ScmpAction::Allow,
-                    syscall,
-                    &[
-                        scmp_cmp!($arg0 <= RawFd::MAX as u64),
-                        scmp_cmp!($arg5 == (*RENAMEAT2_COOKIE_ARG5).into()),
-                    ],
-                )?;
+            // renameat2(2) may be used only with syscall argument cookies.
+            // We also prevent AT_FDCWD usage as fd argument.
+            let sysname = "renameat2";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg0 <= RawFd::MAX as u64),
+                            scmp_cmp!($arg5 == (*RENAMEAT2_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
             }
-            Err(_) => {
-                info!("ctx": "confine", "op": "allow_main_syscall",
-                    "msg": format!("invalid or unsupported syscall {sysname}"));
-            }
-        }
 
-        // unlinkat(2) may be used only with syscall argument cookies.
-        // We also prevent AT_FDCWD usage as fd argument.
-        let sysname = "unlinkat";
-        #[allow(clippy::useless_conversion)]
-        match ScmpSyscall::from_name(sysname) {
-            Ok(syscall) => {
-                ctx.add_rule_conditional(
-                    ScmpAction::Allow,
-                    syscall,
-                    &[
-                        scmp_cmp!($arg0 <= RawFd::MAX as u64),
-                        scmp_cmp!($arg3 == (*UNLINKAT_COOKIE_ARG3).into()),
-                        scmp_cmp!($arg4 == (*UNLINKAT_COOKIE_ARG4).into()),
-                        scmp_cmp!($arg5 == (*UNLINKAT_COOKIE_ARG5).into()),
-                    ],
-                )?;
+            // {,f}truncate{,64}(2) may be used only with syscall argument cookies.
+            let sysname = "truncate";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg2 == (*TRUNCATE_COOKIE_ARG2).into()),
+                            scmp_cmp!($arg3 == (*TRUNCATE_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*TRUNCATE_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*TRUNCATE_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
             }
-            Err(_) => {
-                info!("ctx": "confine", "op": "allow_main_syscall",
-                    "msg": format!("invalid or unsupported syscall {sysname}"));
-            }
-        }
 
-        // openat2(2) may be used only with syscall argument cookies.
-        // We also prevent AT_FDCWD usage as fd argument.
-        let sysname = "openat2";
-        #[allow(clippy::useless_conversion)]
-        match ScmpSyscall::from_name(sysname) {
-            Ok(syscall) => {
+            let sysname = "truncate64";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            #[cfg(target_arch = "x86")]
+                            scmp_cmp!($arg3 == (*TRUNCATE64_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*TRUNCATE64_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*TRUNCATE64_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
+
+            let sysname = "ftruncate";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg2 == (*FTRUNCATE_COOKIE_ARG2).into()),
+                            scmp_cmp!($arg3 == (*FTRUNCATE_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*FTRUNCATE_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*FTRUNCATE_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
+
+            let sysname = "ftruncate64";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            #[cfg(target_arch = "x86")]
+                            scmp_cmp!($arg3 == (*FTRUNCATE64_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*FTRUNCATE64_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*FTRUNCATE64_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
+
+            // unlinkat(2) may be used only with syscall argument cookies.
+            // We also prevent AT_FDCWD usage as fd argument.
+            let sysname = "unlinkat";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg0 <= RawFd::MAX as u64),
+                            scmp_cmp!($arg3 == (*UNLINKAT_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*UNLINKAT_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*UNLINKAT_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
+
+            // openat2(2) may be used only with syscall argument cookies.
+            // We also prevent AT_FDCWD usage as fd argument.
+            let sysname = "openat2";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg0 <= RawFd::MAX as u64),
+                            scmp_cmp!($arg4 == (*OPENAT2_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*OPENAT2_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
+
+            // socket(2) may be used only with syscall argument cookies.
+            // Note: We only enforce this on architectures where the
+            // system call is direct and there's no socketcall(2)
+            // multiplexer indirection.
+            #[allow(clippy::cast_possible_truncation)]
+            #[allow(clippy::useless_conversion)]
+            if let Some(syscall) =
+                SYS_SOCKET.map(|n| ScmpSyscall::from_raw_syscall(n as RawSyscall))
+            {
                 ctx.add_rule_conditional(
                     ScmpAction::Allow,
                     syscall,
                     &[
-                        scmp_cmp!($arg0 <= RawFd::MAX as u64),
-                        scmp_cmp!($arg4 == (*OPENAT2_COOKIE_ARG4).into()),
-                        scmp_cmp!($arg5 == (*OPENAT2_COOKIE_ARG5).into()),
+                        scmp_cmp!($arg3 == (*SOCKET_COOKIE_ARG3).into()),
+                        scmp_cmp!($arg4 == (*SOCKET_COOKIE_ARG4).into()),
+                        scmp_cmp!($arg5 == (*SOCKET_COOKIE_ARG5).into()),
                     ],
                 )?;
+            } else {
+                match ScmpSyscall::from_name("socket") {
+                    Ok(syscall) => {
+                        // Allow socketcall(2).
+                        ctx.add_rule(ScmpAction::Allow, syscall)?;
+                    }
+                    Err(_) => {
+                        info!("ctx": "confine", "op": "allow_emu_syscall",
+                            "msg": "invalid or unsupported syscall socket");
+                    }
+                }
             }
-            Err(_) => {
-                info!("ctx": "confine", "op": "allow_main_syscall",
-                    "msg": format!("invalid or unsupported syscall {sysname}"));
+        } else {
+            // trace/allow_unsafe_nocookie: Allow access without cookies.
+            for sysname in [
+                "memfd_create",
+                "renameat2",
+                "truncate",
+                "truncate64",
+                "ftruncate",
+                "ftruncate64",
+                "unlinkat",
+                "openat2",
+                "socket",
+            ] {
+                match ScmpSyscall::from_name(sysname) {
+                    Ok(syscall) => {
+                        ctx.add_rule(ScmpAction::Allow, syscall)?;
+                    }
+                    Err(_) => {
+                        info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                    }
+                }
             }
         }
 
@@ -724,7 +917,7 @@ impl Worker {
                 )?;
             }
             Err(_) => {
-                info!("ctx": "confine", "op": "allow_syscall",
+                info!("ctx": "confine", "op": "allow_emu_syscall",
                     "msg": format!("invalid or unsupported syscall {sysname}"));
             }
         }
@@ -736,7 +929,7 @@ impl Worker {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -749,7 +942,7 @@ impl Worker {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -769,13 +962,13 @@ impl Worker {
                     )?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
         }
 
-        if safe_setid {
+        if flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID) {
             // SAFETY: Main thread confines these further.
             // As these system calls as per-process,
             // the main thread's seccomp rules will apply
@@ -786,7 +979,7 @@ impl Worker {
                         ctx.add_rule(ScmpAction::Allow, syscall)?;
                     }
                     Err(_) => {
-                        info!("ctx": "confine", "op": "allow_syscall",
+                        info!("ctx": "confine", "op": "allow_emu_syscall",
                             "msg": format!("invalid or unsupported syscall {sysname}"));
                     }
                 }
@@ -802,7 +995,7 @@ impl Worker {
                         ctx.add_rule(ScmpAction::Allow, syscall)?;
                     }
                     Err(_) => {
-                        info!("ctx": "confine", "op": "allow_syscall",
+                        info!("ctx": "confine", "op": "allow_emu_syscall",
                             "msg": format!("invalid or unsupported syscall {sysname}"));
                     }
                 }
@@ -838,19 +1031,17 @@ impl<'a> Sentinel<'a> {
 
     #[allow(clippy::arithmetic_side_effects)]
     fn deny_syscall(&self, seccomp_id: u64, errno: Errno) {
-        let mut resp = libc::seccomp_notif_resp {
+        let response = libc::seccomp_notif_resp {
             id: seccomp_id,
             val: 0,
             error: -(errno as i32),
             flags: 0,
         };
-        // libc's data type is not opaque unlike libseccomp's.
-        let ptr: *mut seccomp_notif_resp = std::ptr::addr_of_mut!(resp) as *mut seccomp_notif_resp;
 
-        // SAFETY:
-        // 1. libseccomp's version allocates needlessly
-        // 2. Nothing we can do on errors, EINTR means child dead.
-        unsafe { seccomp_notify_respond(self.worker_ref.fd, ptr) };
+        // EAGAIN|EINTR is retried.
+        // ENOENT means child died mid-way.
+        // Nothing else we can do on errors here.
+        let _ = seccomp_notify_respond(self.worker_ref.fd, std::ptr::addr_of!(response));
     }
 }
 
