@@ -44,7 +44,7 @@ use crate::{
     info,
     landlock_policy::LandlockPolicy,
     path::{XPath, XPathBuf},
-    sandbox::{Sandbox, LINE_MAX},
+    sandbox::{Flags, Sandbox, LINE_MAX},
 };
 
 const IPC_ACK: &[u8] = b"{\"err\":0,\"msg\":\"ACK\"}\n";
@@ -64,12 +64,8 @@ pub(crate) struct IpcWorker {
     sock: Option<UnixListener>,
     // Reference to the Sandbox to be configured
     sandbox: Option<Arc<RwLock<Sandbox>>>,
-    // State of SafeSetID, used for confinement
-    safe_setid: bool,
-    // State of Speculative Store Bypass mitigations
-    //
-    // If true, mitigations are disabled.
-    scmp_ssb: bool,
+    // Sandbox flags specified at startup.
+    flags: Flags,
 }
 
 // Prompt modes supported (similar to HAProxy)
@@ -149,14 +145,13 @@ impl IpcWorker {
     //
     // This does not bind or listen yet; only stores the path.
     // Returns immediately without error.
-    pub(crate) fn new(addr: &XPath, safe_setid: bool, scmp_ssb: bool) -> Self {
+    pub(crate) fn new(addr: &XPath, flags: Flags) -> Self {
         IpcWorker {
+            flags,
             addr: Some(addr.to_owned()),
             epoll: None,
             sock: None,
             sandbox: None,
-            safe_setid,
-            scmp_ssb,
         }
     }
 
@@ -237,8 +232,7 @@ impl IpcWorker {
     #[allow(clippy::cognitive_complexity)]
     pub(crate) fn prepare_confine(
         epoll_fd: RawFd,
-        safe_setid: bool,
-        ssb: bool,
+        flags: Flags,
         dry_run: bool,
     ) -> SydResult<ScmpFilterContext> {
         if !dry_run {
@@ -267,7 +261,7 @@ impl IpcWorker {
 
         // Disable Speculative Store Bypass mitigations
         // with trace/allow_unsafe_spec_exec:1
-        ctx.set_ctl_ssb(ssb)?;
+        ctx.set_ctl_ssb(flags.contains(Flags::FL_ALLOW_UNSAFE_SPEC_EXEC))?;
 
         // We kill for bad system call and bad arch.
         ctx.set_act_badarch(ScmpAction::KillProcess)?;
@@ -368,7 +362,7 @@ impl IpcWorker {
             }
         }
 
-        if safe_setid {
+        if flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID) {
             // SAFETY: Main thread confines these further.
             // As these system calls as per-process,
             // the main thread's seccomp rules will apply
@@ -423,7 +417,7 @@ impl IpcWorker {
         // Confine `syd_ipc' thread.
         // SAFETY: Panic if we cannot confine the thread.
         #[allow(clippy::disallowed_methods)]
-        let ctx = Self::prepare_confine(epoll.0.as_raw_fd(), self.safe_setid, self.scmp_ssb, false)
+        let ctx = Self::prepare_confine(epoll.0.as_raw_fd(), self.flags, false)
             .expect("prepare confine ipc thread");
         #[allow(clippy::disallowed_methods)]
         ctx.load().expect("confine ipc thread");
